import importlib.util
import os
from typing import Any, Dict, List

import inflection

from . import Request, config

# 全局缓存字典，用于缓存已加载的类
_LOADED_CLASSES_CACHE: Dict[str, List[type]] = {}

# 禁止扫描的目录（防止导入用户虚拟环境）
IGNORE_DIRS = {
    "venv",
    ".venv",
    "env",
    ".env",
    "Lib",
    "lib",
    "site-packages",
    "__pycache__",
}


def load_class(file_path: str, class_name: str) -> Any:
    """从文件加载类"""
    abs_file_path = os.path.abspath(file_path)
    if not os.path.exists(abs_file_path):
        return None
    spec = importlib.util.spec_from_file_location("dynamic_module", file_path)
    if spec is None:
        return None
    module = importlib.util.module_from_spec(spec)
    if module is None:
        return None
    if spec.loader is None:
        return None
    spec.loader.exec_module(module)
    return getattr(module, class_name)


def load_object(file_path: str, class_name: str) -> Any:
    """从文件加载类对象"""
    cls = load_class(file_path, class_name)
    if cls is None:
        return None
    obj = cls()
    return obj


def load_method(file_path: str, class_name: str, method_name: str, *args, **kwargs) -> Any:
    """从文件加载类方法"""
    obj = load_object(file_path, class_name)
    method = getattr(obj, method_name)
    return method(*args, **kwargs)


def get_classes_in_package(package_path: str) -> List[type]:
    """安全扫描指定目录，加载其中的类（带缓存）"""
    global _LOADED_CLASSES_CACHE

    # 检查缓存
    if package_path in _LOADED_CLASSES_CACHE:
        return _LOADED_CLASSES_CACHE[package_path]

    classes = []
    package_dir = os.path.abspath(package_path)

    if not os.path.exists(package_dir):
        _LOADED_CLASSES_CACHE[package_path] = classes
        return classes

    for root, dirs, files in os.walk(package_dir):
        # 过滤不应该扫描的目录
        dirs[:] = [d for d in dirs if d not in IGNORE_DIRS and not d.startswith(".")]

        for file_name in files:
            if not file_name.endswith(".py"):
                continue
            if file_name == "__init__.py":
                continue

            module_path = os.path.join(root, file_name)

            # 构造模块名（安全版本）
            rel_path = os.path.relpath(root, package_dir)
            rel_path = rel_path.replace(os.sep, ".")

            if rel_path == ".":
                full_module_name = file_name[:-3]
            else:
                full_module_name = f"{rel_path}.{file_name[:-3]}"

            # 非法模块名过滤
            if not full_module_name.replace(".", "").isidentifier():
                continue

            try:
                spec = importlib.util.spec_from_file_location(full_module_name, module_path)
                if not spec or not spec.loader:
                    continue

                module = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(module)

                # 遍历类
                for attr_name in dir(module):
                    attr = getattr(module, attr_name)
                    if isinstance(attr, type):
                        classes.append(attr)

            except Exception as e:
                print(f"[WARN] 模块加载失败: {module_path} | {e}")

    # 缓存结果
    _LOADED_CLASSES_CACHE[package_path] = classes
    return classes


def load_resource_classes(class_type: str) -> List[type]:
    """从应用程序目录或quark包加载资源类"""
    get_classes = []

    # 遍历应用目录下的所有类
    app_classes = get_classes_in_package(config.get("MODULE_PATH"))

    # 遍历quark包目录下的所有类
    quark_package_classes = get_classes_in_package(
        os.path.join(os.path.dirname(__file__), "app")
    )

    # 查找指定类名的类
    for getclass in app_classes + quark_package_classes:
        if getclass.__bases__[0].__name__ == class_type:
            get_classes.append(getclass)

    return get_classes


def load_resource(resource: str, class_type: str) -> Any:
    """从应用程序目录或quark包加载资源类对象"""
    app_class = None

    # 遍历应用目录下的所有类
    app_classes = get_classes_in_package(config.get("MODULE_PATH"))

    # 遍历quark包目录下的所有类
    quark_package_classes = get_classes_in_package(
        os.path.join(os.path.dirname(__file__), "app")
    )

    target_name = inflection.camelize(resource)

    # 查找指定类名的类
    for getclass in app_classes + quark_package_classes:
        if getclass.__name__ == target_name and getclass.__bases__[0].__name__ == class_type:
            app_class = getclass
            break

    return app_class


async def load_resource_object(
    request: Request, resource: str, resource_class_type: str
) -> Any:
    """从应用程序目录或quark包加载资源类对象"""
    app_class = load_resource(resource, resource_class_type)
    if app_class is None:
        return None
    return await app_class().init(request)


import json
from typing import Any, Callable, Dict, List, Optional, Union

from pydantic import BaseModel, Field, model_validator

from .base import Base


class FieldNames(BaseModel):
    """
    定义选项中标签、值和子项字段名的类。

    Attributes:
        label (str): 标签字段名。
        value (str): 值字段名。
        children (str): 子项字段名。
    """

    label: str
    value: str
    children: str


class Option(BaseModel):
    """
    表示选择框选项的类。

    Attributes:
        label (str): 选项显示的标签。
        value (Any): 选项对应的值。
        disabled (bool): 选项是否禁用，默认为 False。
    """

    label: str
    value: Any
    disabled: bool = False


class Select(Base):

    component: Optional[str] = "selectField"
    """
    组件名称
    """

    allow_clear: bool = Field(default=True, exclude=True)
    """
    可以点击清除图标删除内容，默认值为 True
    """

    auto_clear_search_value: bool = Field(default=False, exclude=True)
    """
    是否在选中项后清空搜索框，只在 mode 为 multiple 或 tags 时有效，默认值为 False
    """

    auto_focus: bool = Field(default=False, exclude=True)
    """
    默认获取焦点，默认值为 False
    """

    bordered: bool = Field(default=True, exclude=True)
    """
    是否有边框，默认值为 True
    """

    clear_icon: Optional[Any] = Field(default=None, exclude=True)
    """
    自定义的多选框清空图标，默认值为 None
    """

    default_active_first_option: bool = Field(default=True, exclude=True)
    """
    是否默认高亮第一个选项，默认值为 True
    """

    default_open: bool = Field(default=False, exclude=True)
    """
    是否默认展开下拉菜单，默认值为 False
    """

    default_value: Optional[Any] = Field(default=None, exclude=True)
    """
    默认选中的选项，默认值为 None
    """

    disabled: bool = Field(default=False, exclude=True)
    """
    整组失效，默认值为 False
    """

    popup_class_name: str = Field(default="", exclude=True)
    """
    下拉菜单的 className 属性
    """

    dropdown_match_select_width: Optional[Any] = Field(default=None, exclude=True)
    """
    下拉菜单和选择器同宽。默认将设置 min-width，当值小于选择框宽度时会被忽略。false 时会关闭虚拟滚动，默认值为 None
    """

    dropdown_style: Optional[Any] = Field(default=None, exclude=True)
    """
    下拉菜单的 style 属性，默认值为 None
    """

    # 修改第106行代码
    field_names: Optional[FieldNames] = Field(default=None, exclude=True)
    """
    自定义 options 中 label value children 的字段
    """

    label_in_value: bool = Field(default=False, exclude=True)
    """
    是否把每个选项的 label 包装到 value 中，
            会把 Select 的 value 类型从 string 变为 { value: string, label: ReactNode } 的格式，默认值为 False
    """

    list_height: int = Field(default=256, exclude=True)
    """
    设置弹窗滚动高度 256，默认值为 256
    """

    loading: bool = Field(default=False, exclude=True)
    """
    加载中状态，默认值为 False
    """

    max_tag_count: int = Field(default=0, exclude=True)
    """
    最多显示多少个 tag，响应式模式会对性能产生损耗，默认值为 0
    """

    max_tag_placeholder: str = Field(default="", exclude=True)
    """
    隐藏 tag 时显示的内容，默认值为 ""
    """

    max_tag_text_length: int = Field(default=0, exclude=True)
    """
    最大显示的 tag 文本长度，默认值为 0
    """

    menu_item_selected_icon: Optional[Any] = Field(default=None, exclude=True)
    """
    自定义多选时当前选中的条目图标，默认值为 None
    """

    mode: str = Field(default="", exclude=True)
    """
    设置 Select 的模式为多选或标签 multiple | tags，默认值为 ""
    """

    not_found_content: str = Field(default="", exclude=True)
    """
    当下拉列表为空时显示的内容，默认值为 ""
    """

    open: bool = Field(default=False, exclude=True)
    """
    是否展开下拉菜单，默认值为 False
    """

    option_filter_prop: str = Field(default="", exclude=True)
    """
    搜索时过滤对应的 option 属性，如设置为 children 表示对内嵌内容进行搜索。
            若通过 options 属性配置选项内容，建议设置 optionFilterProp="label" 来对内容进行搜索，默认值为 ""
    """

    option_label_prop: str = Field(default="", exclude=True)
    """
    回填到选择框的 Option 的属性值，默认是 Option 的子元素。
            比如在子元素需要高亮效果时，此值可以设为 value，默认值为 ""
    """

    options: List[Option] = Field(default=[])
    """
    可选项数据源，默认值为空列表
    """

    placeholder: str = Field(default="")
    """
    选择框默认文本，默认值为 ""
    """

    placement: str = Field(default="", exclude=True)
    """
    选择框弹出的位置 bottomLeft bottomRight topLeft topRight
    """

    remove_icon: Optional[Any] = Field(default=None, exclude=True)
    """
    自定义的多选框清除图标，默认值为 None
    """

    search_value: str = Field(default="", exclude=True)
    """
    控制搜索文本，默认值为 ""
    """

    show_arrow: bool = Field(default=True, exclude=True)
    """
    是否显示下拉小箭头，默认值为 True
    """

    show_search: bool = Field(default=False, exclude=True)
    """
    配置是否可搜索，默认值为 False
    """

    size: str = Field(default="", exclude=True)
    """
    选择框大小
    """

    status: str = Field(default="", exclude=True)
    """
    设置校验状态 'error' | 'warning'
    """

    suffix_icon: Optional[Any] = Field(default=None, exclude=True)
    """
    自定义的选择框后缀图标，默认值为 None
    """

    token_separators: Optional[Any] = Field(default=None, exclude=True)
    """
    自动分词的分隔符，仅在 mode="tags" 时生效，默认值为 None
    """

    value: Optional[Any] = Field(default=None, exclude=True)
    """
    指定当前选中的条目，多选时为一个数组。
            （value 数组引用未变化时，Select 不会更新），默认值为 None
    """

    virtual: bool = Field(default=True, exclude=True)
    """
    设置 false 时关闭虚拟滚动，默认值为 True
    """

    load: Dict[str, str] = Field(default={}, exclude=True)
    """
    单向联动，默认值为空字典
    """

    style: dict = Field(default_factory=dict)
    """
    自定义样式，默认值为空字典
    """

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        self.set_width("200px")
        return self

    def get_options(self) -> List[Option]:
        """
        获取当前可选项。

        Returns:
            List[Option]: 可选项列表。
        """
        return self.options

    def set_callback(self, closure: Optional[Callable[[Dict[str, Any]], Any]]):
        """
        设置回调函数。

        Args:
            closure (Optional[Callable[[Dict[str, Any]], Any]]): 回调函数。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        if closure is not None:
            self.callback = closure
        return self

    def get_callback(self) -> Any:
        """
        获取回调函数。

        Returns:
            Any: 回调函数。
        """
        return self.callback

    def build_options(
        self, items: Any, label_name: str, value_name: str
    ) -> List[Option]:
        """
        使用反射构建树结构。

        Args:
            items (Any): 包含选项数据的对象。
            label_name (str): 标签字段名。
            value_name (str): 值字段名。

        Returns:
            List[Option]: 构建好的选项列表。
        """
        options: List[Option] = []

        if not isinstance(items, list):
            return options

        for item in items:
            if hasattr(item, label_name) and hasattr(item, value_name):
                label = getattr(item, label_name)
                value = getattr(item, value_name)
                option = Option(label=label, value=value)
                options.append(option)

        return options

    def list_to_options(
        self, list_data: Any, label_name: str, value_name: str
    ) -> List[Option]:
        """
        将列表数据转换为选项列表。

        Args:
            list_data (Any): 列表数据。
            label_name (str): 标签字段名。
            value_name (str): 值字段名。

        Returns:
            List[Option]: 转换后的选项列表。
        """
        return self.build_options(list_data, label_name, value_name)

    def set_options(self, *options: Any):
        """
        设置属性，示例：[Option(value=1, label="男"), Option(value=2, label="女")]
        或者 set_options(options, "label_name", "value_name")

        Args:
            *options: 可变参数，不同数量有不同含义。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        if len(options) == 1:
            self.options = options[0]
            return self
        if len(options) == 3:
            self.options = self.list_to_options(options[0], options[1], options[2])
        return self

    def set_api(self, api: str):
        """
        设置获取数据接口。

        Args:
            api (str): 接口地址。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.api = api
        return self

    def set_allow_clear(self, allow_clear: bool):
        """
        设置是否可以点击清除图标删除内容。

        Args:
            allow_clear (bool): 是否允许清除。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.allow_clear = allow_clear
        return self

    def set_auto_clear_search_value(self, auto_clear_search_value: bool):
        """
        设置是否在选中项后清空搜索框。

        Args:
            auto_clear_search_value (bool): 是否自动清空搜索框。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.auto_clear_search_value = auto_clear_search_value
        return self

    def set_auto_focus(self, auto_focus: bool):
        """
        设置默认是否获取焦点。

        Args:
            auto_focus (bool): 是否自动获取焦点。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.auto_focus = auto_focus
        return self

    def set_bordered(self, bordered: bool):
        """
        设置是否有边框。

        Args:
            bordered (bool): 是否有边框。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.bordered = bordered
        return self

    def set_clear_icon(self, clear_icon: Any):
        """
        设置自定义的多选框清空图标。

        Args:
            clear_icon (Any): 清空图标。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.clear_icon = clear_icon
        return self

    def set_default_active_first_option(self, default_active_first_option: bool):
        """
        设置是否默认高亮第一个选项。

        Args:
            default_active_first_option (bool): 是否默认高亮第一个选项。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.default_active_first_option = default_active_first_option
        return self

    def set_default_open(self, default_open: bool):
        """
        设置是否默认展开下拉菜单。

        Args:
            default_open (bool): 是否默认展开。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.default_open = default_open
        return self

    def set_popup_class_name(self, popup_class_name: str):
        """
        设置下拉菜单的 className 属性。

        Args:
            popup_class_name (str): className 属性值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.popup_class_name = popup_class_name
        return self

    def set_dropdown_match_select_width(self, dropdown_match_select_width: Any):
        """
        设置下拉菜单和选择器是否同宽。

        Args:
            dropdown_match_select_width (Any): 是否同宽。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.dropdown_match_select_width = dropdown_match_select_width
        return self

    def set_dropdown_style(self, dropdown_style: Any):
        """
        设置下拉菜单的 style 属性。

        Args:
            dropdown_style (Any): style 属性值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.dropdown_style = dropdown_style
        return self

    def set_field_names(self, field_names: FieldNames):
        """
        设置自定义 options 中 label value children 的字段。

        Args:
            field_names (FieldNames): 字段名配置。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.field_names = field_names
        return self

    def set_label_in_value(self, label_in_value: bool):
        """
        设置是否把每个选项的 label 包装到 value 中。

        Args:
            label_in_value (bool): 是否包装。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.label_in_value = label_in_value
        return self

    def set_list_height(self, list_height: int):
        """
        设置弹窗滚动高度。

        Args:
            list_height (int): 滚动高度值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.list_height = list_height
        return self

    def set_loading(self, loading: bool):
        """
        设置加载中状态。

        Args:
            loading (bool): 是否加载中。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.loading = loading
        return self

    def set_max_tag_count(self, max_tag_count: int):
        """
        设置最多显示多少个 tag。

        Args:
            max_tag_count (int): 最多显示的 tag 数量。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.max_tag_count = max_tag_count
        return self

    def set_max_tag_placeholder(self, max_tag_placeholder: str):
        """
        设置隐藏 tag 时显示的内容。

        Args:
            max_tag_placeholder (str): 显示内容。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.max_tag_placeholder = max_tag_placeholder
        return self

    def set_max_tag_text_length(self, max_tag_text_length: int):
        """
        设置最大显示的 tag 文本长度。

        Args:
            max_tag_text_length (int): 最大文本长度。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.max_tag_text_length = max_tag_text_length
        return self

    def set_menu_item_selected_icon(self, menu_item_selected_icon: Any):
        """
        设置自定义多选时当前选中的条目图标。

        Args:
            menu_item_selected_icon (Any): 条目图标。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.menu_item_selected_icon = menu_item_selected_icon
        return self

    def set_mode(self, mode: str):
        """
        设置 Select 的模式。

        Args:
            mode (str): 模式，如 'multiple', 'tags'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.mode = mode
        return self

    def set_not_found_content(self, not_found_content: str):
        """
        设置当下拉列表为空时显示的内容。

        Args:
            not_found_content (str): 显示内容。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.not_found_content = not_found_content
        return self

    def set_open(self, open: bool):
        """
        设置是否展开下拉菜单。

        Args:
            open (bool): 是否展开。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.open = open
        return self

    def set_option_filter_prop(self, option_filter_prop: str):
        """
        设置搜索时过滤对应的 option 属性。

        Args:
            option_filter_prop (str): 属性名。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.option_filter_prop = option_filter_prop
        return self

    def set_option_label_prop(self, option_label_prop: str):
        """
        设置回填到选择框的 Option 的属性值。

        Args:
            option_label_prop (str): 属性值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.option_label_prop = option_label_prop
        return self

    def set_placeholder(self, placeholder: str):
        """
        设置选择框默认文本。

        Args:
            placeholder (str): 默认文本。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.placeholder = placeholder
        return self

    def set_placement(self, placement: str):
        """
        设置选择框弹出的位置。

        Args:
            placement (str): 弹出位置，如 'bottomLeft', 'bottomRight' 等。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.placement = placement
        return self

    def set_remove_icon(self, remove_icon: Any):
        """
        设置自定义的多选框清除图标。

        Args:
            remove_icon (Any): 清除图标。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.remove_icon = remove_icon
        return self

    def set_search_value(self, search_value: str):
        """
        设置控制搜索文本。

        Args:
            search_value (str): 搜索文本。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.search_value = search_value
        return self

    def set_show_arrow(self, show_arrow: bool):
        """
        设置是否显示下拉小箭头。

        Args:
            show_arrow (bool): 是否显示。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.show_arrow = show_arrow
        return self

    def set_show_search(self, show_search: bool):
        """
        设置配置是否可搜索。

        Args:
            show_search (bool): 是否可搜索。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.show_search = show_search
        return self

    def set_size(self, size: str):
        """
        设置选择框大小。

        Args:
            size (str): 大小，如 'large', 'middle', 'small'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.size = size
        return self

    def set_status(self, status: str):
        """
        设置校验状态。

        Args:
            status (str): 校验状态，如 'error', 'warning'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.status = status
        return self

    def set_suffix_icon(self, suffix_icon: Any):
        """
        设置自定义的选择框后缀图标。

        Args:
            suffix_icon (Any): 后缀图标。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.suffix_icon = suffix_icon
        return self

    def set_token_separators(self, token_separators: Any):
        """
        设置自动分词的分隔符。

        Args:
            token_separators (Any): 分隔符。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.token_separators = token_separators
        return self

    def set_virtual(self, virtual: bool):
        """
        设置是否关闭虚拟滚动。

        Args:
            virtual (bool): 是否关闭。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.virtual = virtual
        return self

    def set_load(self, field: str, api: str):
        """
        设置单向联动。

        Args:
            field (str): 字段名。
            api (str): 接口地址。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.load = {"field": field, "api": api}
        return self

    def get_value_enum(self) -> Dict[Any, Any]:
        """
        获取当前列值的枚举 valueEnum。

        Returns:
            Dict[Any, Any]: 列值枚举字典。
        """
        data: Dict[Any, Any] = {}
        for option in self.options:
            data[option.value] = option.label
        return data

    def get_option_label(self, value: Any) -> str:
        """
        根据 value 值获取 Option 的 Label。

        Args:
            value (Any): 选项的值。

        Returns:
            str: 对应的标签，多个标签用逗号分隔。
        """
        labels: List[str] = []
        values: List[Any] = []

        if isinstance(value, str):
            if "[" in value or "{" in value:
                try:
                    values = json.loads(value)
                except json.JSONDecodeError:
                    pass

        if values:
            for option in self.options:
                for v in values:
                    if v == option.value:
                        labels.append(option.label)
        else:
            for option in self.options:
                if value == option.value:
                    labels.append(option.label)

        return ",".join(labels)

    def get_option_value(self, label: str) -> Union[List[Any], Any]:
        """
        根据 label 值获取 Option 的 Value。

        Args:
            label (str): 选项的标签，多个标签用逗号或中文逗号分隔。

        Returns:
            Union[List[Any], Any]: 对应的单个值或值列表。
        """
        values: List[Any] = []
        value: Any = None
        labels: List[str] = []

        get_labels = label.split(",")
        if len(get_labels) > 1:
            labels = get_labels
        get_labels = label.split("，")
        if len(get_labels) > 1:
            labels = get_labels

        if len(labels) > 1:
            for option in self.options:
                for get_label in labels:
                    if option.label == get_label:
                        values.append(option.value)
        else:
            for option in self.options:
                if option.label == label:
                    value = option.value

        if values:
            return values
        return value

    def get_option_labels(self) -> str:
        """
        获取所有 Option 的 Labels。

        Returns:
            str: 所有标签，用逗号分隔。
        """
        labels = [option.label for option in self.options]
        return ",".join(labels)

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self



from typing import Any
from fastapi import Request
from tortoise.models import Model
from quark.template.search import Search


class Input(Search):
    """输入框组件"""

    def __init__(self, column: str = "", name: str = ""):
        self.column = column
        self.name = name

    def apply(self, request: Request, query: Model, value: Any) -> Model:
        return query.filter(**{f"{self.column}__icontains": value})



from pydantic import BaseModel
from typing import Any, Dict
from ..component import Component

class View(Component):
    body: Any = None

    # 初始化组件
    @classmethod
    def new(cls) -> 'View':
        return cls().init()

    # 初始化
    def init(self) -> 'View':
        self.element.component = "view"
        self.element.set_key("DEFAULT_KEY", False)
        return self

    # Set style.
    def set_style(self, style: Dict[str, Any]) -> 'View':
        self.element.style = style
        return self

    # 容器控件里面的内容
    def set_body(self, body: Any) -> 'View':
        self.body = body
        return self


from typing import Any, List

from quark import Request, Resource, models
from quark.app import actions, searches
from quark.component.form import field
from quark.component.form.rule import Rule


class Position(Resource):
    """
    职位管理
    """

    async def init(self, request: Request):

        # 页面标题
        self.title = "职位"

        # 模型
        self.model = models.Position

        # 排序字段
        self.index_query_order = ["sort", "id"]

        return self

    async def fields(self, request: Request) -> List[Any]:
        """字段定义"""
        return [
            field.id("id", "ID"),
            field.text("name", "名称").set_rules(
                [
                    Rule.required("职位名称必须填写"),
                    Rule.min(2, "职位名称不能少于2个字符"),
                    Rule.max(100, "职位名称不能超过100个字符"),
                ]
            ),
            field.number("sort", "排序").set_editable(True).set_default_value(0),
            field.textarea("remark", "备注"),
            field.switch("status", "状态")
            .set_editable(True)
            .set_true_value("正常")
            .set_false_value("禁用")
            .set_default_value(True),
        ]

    async def searches(self, request: Request) -> List[Any]:
        """搜索项定义"""
        return [
            searches.Input("name", "名称"),
            searches.Status(),
        ]

    async def actions(self, request: Request) -> List[Any]:
        """行为定义"""
        return [
            actions.CreateModal(self),
            actions.BatchDelete(),
            actions.EditModal(self),
            actions.Delete(),
        ]



from tortoise.exceptions import DoesNotExist

from ..models.user import User
from ..services.attachment import AttachmentService
from ..services.menu import MenuService


class UserService:
    def __init__(self):
        self.attachment_service = AttachmentService()
        self.menu_service = MenuService()

    async def get_info_by_id(self, user_id: int) -> User:
        try:
            user = await User.get(id=user_id, status=1)
            return user
        except DoesNotExist:
            raise Exception("用户不存在")

    async def get_info_by_username(self, username: str) -> User:
        try:
            user = await User.get(username=username, status=1)
            if user.avatar:
                user.avatar = await self.attachment_service.get_image_url(user.avatar)
            return user
        except DoesNotExist:
            raise Exception("用户不存在")

    async def get_menu_list_by_id(self, user_id: int):
        return await self.menu_service.get_list_by_user_id(user_id)

    async def update_last_login(
        self, user_id: int, last_login_ip: str, last_login_time
    ) -> None:
        try:
            user = await User.get(id=user_id)
            user.last_login_ip = last_login_ip
            user.last_login_time = last_login_time
            await user.save()
        except DoesNotExist:
            raise Exception("用户不存在")

    async def count(self) -> int:
        return await User.all().count()



from typing import Any, Dict, Optional

from pydantic import model_validator

from .base import Base


class Number(Base):

    component: str = "inputNumberField"
    """
    组件名称
    """

    addon_after: Optional[Any] = None
    """
    带标签的 input，设置后置标签
    """

    addon_before: Optional[Any] = None
    """
    带标签的 input，设置前置标签
    """

    allow_clear: bool = False
    """
    可以点击清除图标删除内容，默认值为 False
    """

    bordered: bool = True
    """
    是否有边框，默认值为 True
    """

    controls: bool = False
    """
    是否显示增减按钮，也可设置自定义箭头图标，默认值为 False
    """

    decimal_separator: Optional[str] = None
    """
    小数点，默认值为 None
    """

    default_value: Optional[Any] = None
    """
    默认的选中项
    """

    disabled: Optional[Any] = None
    """
    禁用
    """

    keyboard: bool = False
    """
    是否启用键盘快捷行为，默认值为 False
    """

    max: Optional[int] = None
    """
    最大值
    """

    min: Optional[int] = None
    """
    最小值
    """

    precision: Optional[int] = None
    """
    数值精度，配置 formatter 时会以 formatter 为准
    """

    read_only: bool = False
    """
    只读，默认值为 False
    """

    status: Optional[str] = None
    """
    设置校验状态,'error' | 'warning'
    """

    prefix: Optional[Any] = None
    """
    带有前缀图标的 input
    """

    size: Optional[str] = None
    """
    控件大小。注：标准表单内的输入框大小限制为 middle，large | middle | small
    """

    step: Optional[Any] = None
    """
    每次改变步数，可以为小数
    """

    string_mode: bool = False
    """
    字符值模式，开启后支持高精度小数。同时 onChange 将返回 string 类型，默认值为 False
    """

    value: Optional[Any] = None
    """
    指定选中项
    """

    placeholder: str = "请输入"
    """
    占位符，默认为 "请输入"
    """

    style: Dict[str, Any] = {}
    """
    自定义样式
    """

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        self.set_width("200px")
        return self

    def set_addon_after(self, addon_after: Any):
        """
        设置带标签的 input 后置标签。

        Args:
            addon_after (Any): 后置标签信息。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.addon_after = addon_after
        return self

    def set_addon_before(self, addon_before: Any):
        """
        设置带标签的 input 前置标签。

        Args:
            addon_before (Any): 前置标签信息。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.addon_before = addon_before
        return self

    def set_allow_clear(self, allow_clear: bool):
        """
        设置是否可以点击清除图标删除内容。

        Args:
            allow_clear (bool): 是否允许清除。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.allow_clear = allow_clear
        return self

    def set_bordered(self, bordered: bool):
        """
        设置是否有边框。

        Args:
            bordered (bool): 是否有边框。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.bordered = bordered
        return self

    def set_controls(self, controls: bool):
        """
        设置是否显示增减按钮。

        Args:
            controls (bool): 是否显示按钮。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.controls = controls
        return self

    def set_decimal_separator(self, decimal_separator: str):
        """
        设置小数点。

        Args:
            decimal_separator (str): 小数点字符。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.decimal_separator = decimal_separator
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self

    def set_keyboard(self, keyboard: bool):
        """
        设置是否启用键盘快捷行为。

        Args:
            keyboard (bool): 是否启用。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.keyboard = keyboard
        return self

    def set_max(self, max: int):
        """
        设置最大值。

        Args:
            max (int): 最大值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.max = max
        return self

    def set_min(self, min: int):
        """
        设置最小值。

        Args:
            min (int): 最小值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.min = min
        return self

    def set_precision(self, precision: int):
        """
        设置数值精度。

        Args:
            precision (int): 精度值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.precision = precision
        return self

    def set_read_only(self, read_only: bool):
        """
        设置只读状态。

        Args:
            read_only (bool): 是否只读。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.read_only = read_only
        return self

    def set_status(self, status: str):
        """
        设置校验状态。

        Args:
            status (str): 状态，如 'error', 'warning'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.status = status
        return self

    def set_placeholder(self, placeholder: str):
        """
        设置输入框占位文本。

        Args:
            placeholder (str): 占位文本。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.placeholder = placeholder
        return self

    def set_prefix(self, prefix: Any):
        """
        设置带有前缀图标的 input。

        Args:
            prefix (Any): 前缀图标信息。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.prefix = prefix
        return self

    def set_size(self, size: str):
        """
        设置控件大小。

        Args:
            size (str): 大小，如 'large', 'middle', 'small'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.size = size
        return self

    def set_step(self, step: Any):
        """
        设置每次改变步数。

        Args:
            step (Any): 步数，可以为小数。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.step = step
        return self

    def set_string_mode(self, string_mode: bool):
        """
        设置字符值模式。

        Args:
            string_mode (bool): 是否开启字符值模式。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.string_mode = string_mode
        return self



from typing import Any, Dict, List, Optional

from .base import Base


class FieldNames(Base):
    """
    表示 Tree 组件字段名映射的类。

    Attributes:
        title (str): 对应树节点标题的字段名。
        key (str): 对应树节点键的字段名。
        children (str): 对应树节点子节点的字段名。
    """

    title: str
    key: str
    children: str


class TreeData(Base):
    """
    表示 Tree 组件树节点数据的类。

    Attributes:
        checkable (bool): 当树为 checkable 时，设置独立节点是否展示 Checkbox。
        disable_checkbox (bool): 禁掉 checkbox。
        disabled (bool): 禁掉响应。
        icon (Optional[Any]): 自定义图标。可接收组件，props 为当前节点 props。
        is_leaf (bool): 设置为叶子节点 (设置了 loadData 时有效)。为 false 时会强制将其作为父节点。
        key (Any): 被树的 (default)ExpandedKeys / (default)CheckedKeys / (default)SelectedKeys 属性所用。
            注意：整个树范围内的所有节点的 key 值不能重复！
        selectable (bool): 设置节点是否可被选中。
        title (str): 标题。
        children (List['TreeData']): 子节点。
    """

    checkable: bool = None
    disable_checkbox: bool = None
    disabled: bool = None
    icon: Optional[Any] = None
    is_leaf: bool = None
    key: Any
    selectable: bool = None
    title: str
    children: List["TreeData"] = []


class Tree(Base):

    component: str = "treeField"
    """
    组件名称
    """

    auto_expand_parent: bool = False
    """
    是否自动展开父节点，默认值为 False
    """

    block_node: bool = False
    """
    是否节点占据一行，默认值为 False
    """

    checkable: bool = True
    """
    节点前添加 Checkbox 复选框，默认值为 True
    """

    checked_keys: List[Any] = []
    """
    （受控）选中复选框的树节点，默认值为 []
    """

    check_strictly: bool = False
    """
    checkable 状态下节点选择完全受控（父子节点选中状态不再关联），默认值为 False
    """

    default_checked_keys: List[Any] = []
    """
    默认选中复选框的树节点，默认值为 []
    """

    default_expand_all: bool = False
    """
    默认展开所有树节点，默认值为 False
    """

    default_expanded_keys: List[Any] = []
    """
    默认展开指定的树节点，默认值为 []
    """

    default_expand_parent: bool = False
    """
    默认展开父节点，默认值为 False
    """

    default_selected_keys: List[Any] = []
    """
    默认选中的树节点，默认值为 []
    """

    default_value: Optional[Any] = None
    """
    默认选中的选项，默认值为 None
    """

    disabled: bool = False
    """
    整组失效，默认值为 False
    """

    draggable: bool = False
    """
    设置节点可拖拽，可以通过 icon: false 关闭拖拽提示图标，默认值为 False
    """

    expanded_keys: List[Any] = []
    """
    （受控）展开指定的树节点，默认值为 []
    """

    field_names: Optional[FieldNames] = None
    """
    自定义 options 中 label value children 的字段，默认值为 None
    """

    height: int = None
    """
    设置虚拟滚动容器高度，设置后内部节点不再支持横向滚动
    """

    icon: Optional[Any] = None
    """
    自定义树节点图标
    """

    multiple: bool = False
    """
    支持点选多个节点（节点本身），默认值为 False
    """

    placeholder: Optional[str] = None
    """
    占位文本，默认值为 None
    """

    root_class_name: str = None
    """
    添加在 Tree 最外层的 className，默认值为 ""
    """

    root_style: Optional[Any] = None
    """
    添加在 Tree 最外层的 style，默认值为 None
    """

    selectable: bool = True
    """
    是否可选中，默认值为 True
    """

    selected_keys: List[Any] = []
    """
    （受控）设置选中的树节点，默认值为 []
    """

    show_icon: bool = False
    """
    是否展示 TreeNode title 前的图标，没有默认样式，
            如设置为 true，需要自行定义图标相关样式，默认值为 False
    """

    show_line: bool = False
    """
    是否展示连接线，默认值为 False
    """

    switcher_icon: Optional[Any] = None
    """
    自定义树节点的展开/折叠图标，默认值为 None
    """

    tree_data: List[TreeData] = []
    """
    treeNodes 数据，如果设置则不需要手动构造 TreeNode 节点，默认值为 []
    """

    value: Optional[Any] = None
    """
    指定当前选中的条目，多选时为一个数组，默认值为 None
    """

    virtual: bool = True
    """
    设置 false 时关闭虚拟滚动，默认值为 True
    """

    style: Optional[Dict[str, Any]] = None
    """
    自定义样式，默认值为 None
    """

    def get_options(self) -> List[TreeData]:
        """
        获取当前可选项。

        Returns:
            List[TreeData]: 当前可选项列表。
        """
        return self.tree_data

    def set_auto_expand_parent(self, auto_expand_parent: bool):
        """
        设置是否自动展开父节点。

        Args:
            auto_expand_parent (bool): 是否自动展开父节点。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.auto_expand_parent = auto_expand_parent
        return self

    def set_block_node(self, block_node: bool):
        """
        设置是否节点占据一行。

        Args:
            block_node (bool): 是否节点占据一行。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.block_node = block_node
        return self

    def set_checkable(self, checkable: bool):
        """
        设置节点前是否添加 Checkbox 复选框。

        Args:
            checkable (bool): 是否添加复选框。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.checkable = checkable
        return self

    def set_checked_keys(self, checked_keys: List[Any]):
        """
        设置（受控）选中复选框的树节点。

        Args:
            checked_keys (List[Any]): 选中的树节点 key 列表。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.checked_keys = checked_keys
        return self

    def set_check_strictly(self, check_strictly: bool):
        """
        设置 checkable 状态下节点选择是否完全受控。

        Args:
            check_strictly (bool): 是否完全受控。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.check_strictly = check_strictly
        return self

    def set_default_checked_keys(self, default_checked_keys: List[Any]):
        """
        设置默认选中复选框的树节点。

        Args:
            default_checked_keys (List[Any]): 默认选中的树节点 key 列表。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.default_checked_keys = default_checked_keys
        return self

    def set_default_expand_all(self, default_expand_all: bool):
        """
        设置默认是否展开所有树节点。

        Args:
            default_expand_all (bool): 是否展开所有树节点。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.default_expand_all = default_expand_all
        return self

    def set_default_expanded_keys(self, default_expanded_keys: List[Any]):
        """
        设置默认展开指定的树节点。

        Args:
            default_expanded_keys (List[Any]): 默认展开的树节点 key 列表。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.default_expanded_keys = default_expanded_keys
        return self

    def set_default_expand_parent(self, default_expand_parent: bool):
        """
        设置默认是否展开父节点。

        Args:
            default_expand_parent (bool): 是否展开父节点。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.default_expand_parent = default_expand_parent
        return self

    def set_default_selected_keys(self, default_selected_keys: List[Any]):
        """
        设置默认选中的树节点。

        Args:
            default_selected_keys (List[Any]): 默认选中的树节点 key 列表。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.default_selected_keys = default_selected_keys
        return self

    def set_draggable(self, draggable: bool):
        """
        设置节点是否可拖拽。

        Args:
            draggable (bool): 是否可拖拽。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.draggable = draggable
        return self

    def set_expanded_keys(self, expanded_keys: List[Any]):
        """
        设置（受控）展开指定的树节点。

        Args:
            expanded_keys (List[Any]): 展开的树节点 key 列表。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.expanded_keys = expanded_keys
        return self

    def set_field_names(self, field_names: Optional[FieldNames]):
        """
        设置自定义 options 中 label value children 的字段。

        Args:
            field_names (Optional[FieldNames]): 字段名映射对象。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.field_names = field_names
        return self

    def set_height(self, height: int):
        """
        设置虚拟滚动容器高度。

        Args:
            height (int): 容器高度。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.height = height
        return self

    def set_icon(self, icon: Any):
        """
        设置自定义树节点图标。

        Args:
            icon (Any): 树节点图标。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.icon = icon
        return self

    def set_multiple(self, multiple: bool):
        """
        设置是否支持点选多个节点。

        Args:
            multiple (bool): 是否支持多选。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.multiple = multiple
        return self

    def set_placeholder(self, placeholder: str):
        """
        设置占位文本。

        Args:
            placeholder (str): 占位文本内容。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.placeholder = placeholder
        return self

    def set_root_class_name(self, root_class_name: str):
        """
        设置添加在 Tree 最外层的 className。

        Args:
            root_class_name (str): className 名称。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.root_class_name = root_class_name
        return self

    def set_root_style(self, root_style: Any):
        """
        设置添加在 Tree 最外层的 style。

        Args:
            root_style (Any): 样式信息。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.root_style = root_style
        return self

    def set_selectable(self, selectable: bool):
        """
        设置节点是否可选中。

        Args:
            selectable (bool): 是否可选中。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.selectable = selectable
        return self

    def set_selected_keys(self, selected_keys: List[Any]):
        """
        设置（受控）选中的树节点。

        Args:
            selected_keys (List[Any]): 选中的树节点 key 列表。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.selected_keys = selected_keys
        return self

    def set_show_icon(self, show_icon: bool):
        """
        设置是否展示 TreeNode title 前的图标。

        Args:
            show_icon (bool): 是否展示图标。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.show_icon = show_icon
        return self

    def set_show_line(self, show_line: bool):
        """
        设置是否展示连接线。

        Args:
            show_line (bool): 是否展示连接线。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.show_line = show_line
        return self

    def set_switcher_icon(self, switcher_icon: Any):
        """
        设置自定义树节点的展开/折叠图标。

        Args:
            switcher_icon (Any): 展开/折叠图标。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.switcher_icon = switcher_icon
        return self

    def build_tree(
        self, items: Any, pid: int, parent_key_name: str, key_name: str, title_name: str
    ) -> List[TreeData]:
        """
        使用反射构建树结构。

        Args:
            items (Any): 包含树节点数据的对象。
            pid (int): 父节点 ID。
            parent_key_name (str): 父节点键名。
            key_name (str): 节点键名。
            title_name (str): 节点标题名。

        Returns:
            List[TreeData]: 构建好的树节点数据列表。
        """
        tree = []

        for item in items:
            # 支持对象和字典两种格式
            if hasattr(item, key_name):
                key = getattr(item, key_name)
                parent_key = getattr(item, parent_key_name)
                title = getattr(item, title_name)
            elif isinstance(item, dict):
                key = item.get(key_name)
                parent_key = item.get(parent_key_name)
                title = item.get(title_name)
            else:
                # 如果既不是对象也不是字典，跳过该项
                continue

            if parent_key == pid:
                children = self.build_tree(
                    items, key, parent_key_name, key_name, title_name
                )
                option = TreeData(title=title, key=key, children=children)
                tree.append(option)

        return tree

    def list_to_tree_data(
        self,
        list_: Any,
        root_id: int,
        parent_key_name: str,
        key_name: str,
        title_name: str,
    ) -> List[TreeData]:
        """
        将列表转换为树节点数据。

        Args:
            list_ (Any): 包含树节点数据的列表。
            root_id (int): 根节点 ID。
            parent_key_name (str): 父节点键名。
            key_name (str): 节点键名。
            title_name (str): 节点标题名。

        Returns:
            List[TreeData]: 转换后的树节点数据列表。
        """
        return self.build_tree(list_, root_id, parent_key_name, key_name, title_name)

    def set_tree_data(self, *tree_data: Any):
        """
        设置可选项数据源。

        Args:
            *tree_data: 可变参数，不同数量有不同含义。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        if len(tree_data) == 1:
            if isinstance(tree_data[0], list) and all(
                isinstance(item, TreeData) for item in tree_data[0]
            ):
                self.tree_data = tree_data[0]
        elif len(tree_data) == 4:
            self.tree_data = self.list_to_tree_data(
                tree_data[0], 0, tree_data[1], tree_data[2], tree_data[3]
            )
        elif len(tree_data) == 5:
            self.tree_data = self.list_to_tree_data(
                tree_data[0], tree_data[1], tree_data[2], tree_data[3], tree_data[4]
            )
        return self

    def set_style(self, style: Dict[str, Any]):
        """
        设置自定义样式。

        Args:
            style (Dict[str, Any]): 样式字典。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.style = style
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self



import json
from typing import List, Optional

from tortoise.expressions import Q
from tortoise.queryset import QuerySet

from quark import Request


class PerformsQueries:

    # 请求对象
    request: Request

    # 查询对象
    query: QuerySet

    # 全局查询排序
    query_order: List[str]

    # 列表查询排序
    index_query_order: List[str]

    # 导出查询排序
    export_query_order: List[str]

    def __init__(
        self,
        request: Optional[Request] = None,
        query: Optional[QuerySet] = None,
        query_order: Optional[List[str]] = None,
        index_query_order: Optional[List[str]] = None,
        export_query_order: Optional[List[str]] = None,
    ):
        self.request = request
        self.query = query
        self.query_order = query_order
        self.index_query_order = index_query_order
        self.export_query_order = export_query_order

    # 创建行为查询
    def build_action_query(self, query: QuerySet) -> QuerySet:
        return self.action_query(query)

    # 创建详情页查询
    def build_detail_query(self, query: QuerySet) -> QuerySet:
        return self.detail_query(query)

    # 创建编辑页查询
    def build_edit_query(self, query: QuerySet) -> QuerySet:
        return self.edit_query(query)

    # 创建表格行内编辑查询
    def build_editable_query(self, query: QuerySet) -> QuerySet:
        return self.editable_query(query)

    # 创建导出查询
    def build_export_query(self, search, column_filters) -> QuerySet:
        query = self.export_query(self.query)
        query = self.apply_search(query, search)
        query = self.apply_column_filters(query, column_filters)
        return query

    # 创建列表查询
    def build_index_query(self, search, column_filters) -> QuerySet:
        query = self.index_query(self.query)
        query = self.apply_search(query, search)
        query = self.apply_column_filters(query, column_filters)
        return query

    # 创建更新查询
    async def build_update_query(self, query: QuerySet) -> QuerySet:
        return await self.update_query(query)

    # 执行搜索表单查询
    def apply_search(self, query: QuerySet, search) -> QuerySet:
        json_string = self.request.query_params.get("search", {})
        if not json_string:
            return query

        data = json.loads(json_string)
        for v in search:
            column = v.get_column(v)
            value = data.get(column)
            if value is not None:
                query = v.apply(self.request, query, value)

        return query

    # 执行表格列上过滤器查询
    def apply_column_filters(self, query: QuerySet, filters) -> QuerySet:
        if not filters:
            return query
        for k, v in filters.items():
            if v is not None:
                query = query.filter(getattr(query, k).in_(v))
        return query

    def apply_index_orderings(self, query: QuerySet, orderings) -> QuerySet:
        default_order = self.query_order
        if not default_order or len(default_order) == 0:
            default_order = self.index_query_order
        if not default_order or len(default_order) == 0:
            default_order = ["-id"]

        return self.apply_orderings(query, orderings, default_order)

    def apply_export_orderings(self, query: QuerySet, orderings) -> QuerySet:
        default_order = self.query_order
        if not default_order or len(default_order) == 0:
            default_order = self.export_query_order
        if not default_order or len(default_order) == 0:
            default_order = ["-id"]
        return self.apply_orderings(query, orderings, default_order)

    # 执行排序查询
    def apply_orderings(
        self, query: QuerySet, orderings, default_order: List[str]
    ) -> QuerySet:
        if not orderings:
            return query.order_by(*default_order)
        for key, v in orderings.items():
            if v == "descend":
                query = query.order_by(getattr(query, key).desc())
            else:
                query = query.order_by(getattr(query, key).asc())
        return query

    # 行为查询
    def action_query(self, query: QuerySet) -> QuerySet:
        id_param = self.request.query_params.get("id")
        if id_param:
            ids = id_param.split(",") if "," in id_param else [id_param]
            query = query.filter(Q(id__in=ids))
        return query

    # 详情查询
    def detail_query(self, query: QuerySet) -> QuerySet:
        id_param = self.request.query_params.get("id")
        if id_param:
            query = query.filter(Q(id=id_param))
        return query

    # 编辑查询
    def edit_query(self, query: QuerySet) -> QuerySet:
        id_param = self.request.query_params.get("id")
        if id_param:
            query = query.filter(Q(id=id_param))
        return query

    # 表格行内编辑查询
    def editable_query(self, query: QuerySet) -> QuerySet:
        id_param = self.request.query_params.get("id")
        if id_param:
            query = query.filter(Q(id=id_param))
        return query

    # 导出查询
    def export_query(self, query: QuerySet) -> QuerySet:
        id_param = self.request.query_params.get("id")
        if id_param:
            ids = id_param.split(",") if "," in id_param else [id_param]
            query = query.filter(Q(id__in=ids))
        return query

    # 列表查询
    def index_query(self, query: QuerySet) -> QuerySet:
        return query

    # 更新查询
    async def update_query(self, query: QuerySet) -> QuerySet:
        data = await self.request.json() or {}
        if "id" in data:
            query = query.filter(id=data["id"])
        return query



from quark import Request
from quark.template.action import Link


class CreateLink(Link):

    def __init__(self, title: str = ""):

        # 文字
        self.name = f"创建{title}"

        # 类型
        self.type = "primary"

        # 图标
        self.icon = "plus-circle"

        # 设置只在索引页显示
        self.set_only_on_index(True)

    async def get_href(self, request: Request) -> str:
        return "#/layout/index?api=" + request.url.path.replace("/index", "/create")



from pydantic import model_validator
from typing import Any, Callable, Optional
from ..component import Component
from ..modal.modal import Modal
from ..drawer.drawer import Drawer


class Item(Component):
    component: str = "menuItem"
    title: str = ""
    label: str = ""
    block: bool = False
    danger: bool = False
    disabled: bool = False
    ghost: bool = False
    icon: str = ""
    shape: str = ""
    size: str = "default"
    type: str = "default"
    action_type: str = ""
    submit_form: Optional[Any] = None
    href: str = ""
    target: str = ""
    modal: Optional[Any] = None
    drawer: Optional[Any] = None
    confirm_title: str = ""
    confirm_text: str = ""
    confirm_type: str = ""
    api: str = ""
    reload: str = ""
    with_loading: bool = False

    @model_validator(mode="after")
    def init(self):
        self.set_key("DEFAULT_KEY", False)
        return self

    # Set style.
    def set_style(self, style: dict[str, Any]) -> "Item":
        self.element.style = style
        return self

    # 设置收缩时展示的悬浮标题
    def set_title(self, title: str) -> "Item":
        self.title = title
        return self

    # 设置按钮文字
    def set_label(self, label: str) -> "Item":
        self.label = label
        return self

    # 将按钮宽度调整为其父宽度的选项
    def set_block(self, block: bool) -> "Item":
        self.block = block
        return self

    # 设置危险按钮
    def set_danger(self, danger: bool) -> "Item":
        self.danger = danger
        return self

    # 按钮失效状态
    def set_disabled(self, disabled: bool) -> "Item":
        self.disabled = disabled
        return self

    # 幽灵属性，使按钮背景透明
    def set_ghost(self, ghost: bool) -> "Item":
        self.ghost = ghost
        return self

    # 设置按钮图标
    def set_icon(self, icon: str) -> "Item":
        self.icon = "icon-" + icon
        return self

    # 设置按钮形状，可选值为 circle、 round 或者不设
    def set_shape(self, shape: str) -> "Item":
        self.shape = shape
        return self

    # 设置按钮类型，primary | ghost | dashed | link | text | default
    def set_type(self, button_type: str, danger: bool) -> "Item":
        self.type = button_type
        self.danger = danger
        return self

    # 设置按钮大小，large | middle | small | default
    def set_size(self, size: str) -> "Item":
        self.size = size
        return self

    # 【必填】这是 action 最核心的配置，来指定该 action 的作用类型，支持：ajax、link、url、drawer、dialog、confirm、cancel、prev、next、copy、close。
    def set_action_type(self, action_type: str) -> "Item":
        self.action_type = action_type
        return self

    # 当action 的作用类型为submit的时候，可以指定提交哪个表格，submitForm为提交表单的key值，为空时提交当前表单
    def set_submit_form(self, form_key: Any) -> "Item":
        self.submit_form = form_key
        return self

    # 点击跳转的地址，指定此属性 button 的行为和 a 链接一致
    def set_href(self, href: str) -> "Item":
        self.href = href
        return self

    # 相当于 a 链接的 target 属性，href 存在时生效
    def set_target(self, target: str) -> "Item":
        self.target = target
        return self

    # 设置跳转链接
    def set_link(self, href: str, target: str) -> "Item":
        self.set_href(href)
        self.set_target(target)
        self.action_type = "link"
        return self

    # 弹窗
    def set_modal(self, callback: Callable[[Modal], Any]) -> "Item":
        component = Modal()
        self.modal = callback(component)
        return self

    # 抽屉
    def set_drawer(self, callback: Callable[[Drawer], Any]) -> "Item":
        component = Drawer()
        self.drawer = callback(component)
        return self

    # 设置行为前的确认操作
    def set_with_confirm(self, title: str, text: str, confirm_type: str) -> "Item":
        self.confirm_title = title
        self.confirm_text = text
        self.confirm_type = confirm_type
        return self

    # 执行行为的接口链接
    def set_api(self, api: str) -> "Item":
        self.api = api
        self.action_type = "ajax"
        return self

    # 执行成功后刷新的组件
    def set_reload(self, reload: str) -> "Item":
        self.reload = reload
        return self

    # 是否具有loading
    def set_with_loading(self, loading: bool) -> "Item":
        self.with_loading = loading
        return self



import json
from typing import Any, Callable, Dict, List, Optional

from pydantic import Field, model_validator

from ...component import Component
from ...table.column import Column
from ..rule import Rule
from .when import Item, When


class Base(Component):

    row_props: Optional[Dict[str, Any]] = None
    """
    开启 grid 模式时传递给 Row, 仅在ProFormGroup, ProFormList, ProFormFieldSet 中有效，默认：{ gutter: 8 }
    """

    col_props: Optional[Dict[str, Any]] = None
    """
    开启 grid 模式时传递给 Col，默认：{ xs: 24 }
    """

    secondary: bool = False
    """
    是否是次要控件，只针对 LightFilter 下有效
    """

    colon: bool = True
    """
    配合 label 属性使用，表示是否显示 label 后面的冒号
    """

    extra: str = ""
    """
    额外的提示信息，和 help 类似，当需要错误信息和提示文案同时出现时，可以使用这个。
    """

    has_feedback: bool = False
    """
    配合 valiTextStatus 属性使用，展示校验状态图标，建议只配合 Input 组件使用
    """

    help: Optional[str] = None
    """
    提示信息，如不设置，则会根据校验规则自动生成
    """

    hidden: bool = False
    """
    是否隐藏字段（依然会收集和校验字段）
    """

    initial_value: Optional[Any] = None
    """
    设置子元素默认值，如果与 Form 的 initialValues 冲突则以 Form 为准
    """

    label: str = ""
    """
    label 标签的文本
    """

    label_align: str = "right"
    """
    标签文本对齐方式
    """

    label_col: Optional[Any] = None
    """
    label 标签布局，同 <Col> 组件，设置 span offset 值，如 {span: 3, offset: 12} 或 sm: {span: 3, offset: 12}。你可以通过 Form 的 labelCol 进行统一设置，不会作用于嵌套 Item。当和 Form 同时设置时，以 Item 为准
    """

    name: str = ""
    """
    字段名，支持数组
    """

    no_style: bool = False
    """
    为 true 时不带样式，作为纯字段控件使用
    """

    required: Optional[bool] = None
    """
    必填样式设置。如不设置，则会根据校验规则自动生成
    """

    tooltip: Optional[str] = None
    """
    会在 label 旁增加一个 icon，悬浮后展示配置的信息
    """

    value_prop_name: Optional[str] = None
    """
    子节点的值的属性，如 Switch 的是 'checked'。该属性为 getValueProps 的封装，自定义 getValueProps 后会失效
    """

    wrapper_col: Optional[Any] = None
    """
    需要为输入控件设置布局样式时，使用该属性，用法同 labelCol。你可以通过 Form 的 wrapperCol 进行统一设置，不会作用于嵌套 Item。当和 Form 同时设置时，以 Item 为准
    """

    column: Column = Field(default_factory=lambda: Column())
    """
    列表页、详情页中列属性
    """

    align: Optional[str] = None
    """
    设置列的对齐方式,left | right | center，只在列表页、详情页中有效
    """

    fixed: Optional[Any] = None
    """
    （IE 下无效）列是否固定，可选 true (等效于 left) left rightr，只在列表页中有效
    """

    editable: bool = False
    """
    表格列是否可编辑，只在列表页中有效
    """

    ellipsis: bool = False
    """
    是否自动缩略，只在列表页、详情页中有效
    """

    copyable: bool = False
    """
    是否支持复制，只在列表页、详情页中有效
    """

    filters: Optional[Any] = None
    """
    表头的筛选菜单项，当值为 true 时，自动使用 valueEnum 生成，只在列表页中有效
    """

    order: Optional[int] = None
    """
    查询表单中的权重，权重大排序靠前，只在列表页中有效
    """

    sorter: Optional[Any] = None
    """
    可排序列，只在列表页中有效
    """

    span: Optional[int] = None
    """
    包含列的数量，只在详情页中有效
    """

    column_width: Optional[int] = None
    """
    设置列宽，只在列表页中有效
    """

    api: str = ""
    """
    获取数据接口
    """

    ignore: bool = False
    """
    是否忽略保存到数据库，默认为 false
    """

    rules: List[Rule] = []
    """
    全局校验规则
    """

    creation_rules: List[Rule] = []
    """
    创建页校验规则
    """

    update_rules: List[Rule] = []
    """
    编辑页校验规则
    """

    frontend_rules: List[Rule] = []
    """
    前端校验规则，设置字段的校验逻辑
    """

    when: Optional[When] = None
    """
    When 组件
    """

    when_item: List[Item] = Field(default_factory=list)
    """
    When 组件的项
    """

    show_on_index: bool = True
    """
    在列表页展示
    """

    show_on_detail: bool = True
    """
    在详情页展示
    """

    show_on_creation: bool = True
    """
    在创建页面展示
    """

    show_on_update: bool = True
    """
    在编辑页面展示
    """

    show_on_export: bool = True
    """
    在导出的Excel上展示
    """

    show_on_import: bool = True
    """
    在导入Excel上展示
    """

    callback: Optional[Callable[[Dict[str, Any]], Any]] = Field(
        exclude=True, default=None
    )
    """
    回调函数
    """

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        return self

    def set_style(self, style: Dict[str, Any]):
        """设置组件的样式。"""
        # 直接使用父类的style属性
        current_style = getattr(self, "style", {}) or {}
        if hasattr(self, "style"):
            self.style = {**current_style, **style}
        else:
            # 如果父类没有style属性，则设置
            self.__dict__["style"] = style
        return self

    def set_tooltip(self, tooltip: str):
        """设置 tooltip 信息。"""
        self.tooltip = tooltip
        return self

    def set_width(self, width: Any):
        """设置组件的宽度。"""
        style = self.style.copy() if self.style else {}
        style["width"] = width
        self.style = style
        return self

    def set_row_props(self, row_props: Dict[str, Any]):
        """设置 Row 属性。"""
        self.row_props = row_props
        return self

    def set_col_props(self, col_props: Dict[str, Any]):
        """设置 Col 属性。"""
        self.col_props = col_props
        return self

    def set_secondary(self, secondary: bool):
        """设置是否为次要控件。"""
        self.secondary = secondary
        return self

    def set_colon(self, colon: bool):
        """设置是否显示 label 后面的冒号。"""
        self.colon = colon
        return self

    def set_extra(self, extra: str):
        """设置额外的提示信息。"""
        self.extra = extra
        return self

    def set_has_feedback(self, has_feedback: bool):
        """设置是否展示校验状态图标。"""
        self.has_feedback = has_feedback
        return self

    def set_help(self, help: str):
        """设置提示信息。"""
        self.help = help
        return self

    def set_no_style(self):
        """设置为纯字段控件。"""
        self.no_style = True
        return self

    def set_label(self, label: str):
        """设置 label 标签的文本。"""
        self.label = label
        return self

    def set_label_align(self, align: str):
        """设置标签文本对齐方式。"""
        self.label_align = align
        return self

    def set_label_col(self, col: Any):
        """设置 label 标签布局。"""
        self.label_col = col
        return self

    def set_name(self, name: str):
        """设置字段名。"""
        self.name = name
        return self

    def set_name_as_label(self):
        """将字段名转换为 label。"""
        self.label = self.name.title()
        return self

    def set_required(self):
        """设置字段为必填。"""
        self.required = True
        return self

    def build_frontend_rules(self, path: str):
        rules = self.rules
        uri = path.split("/")
        is_creating = uri[-1] in ["create", "store"]
        is_editing = uri[-1] in ["edit", "update"]

        if is_creating:
            rules.extend(self.creation_rules)
        if is_editing:
            rules.extend(self.update_rules)

        self.frontend_rules = Rule.convert_to_frontend_rules(rules)
        return self

    def set_rules(self, rules: List[Rule]):
        for rule in rules:
            rule.name = self.name
        self.rules = rules
        return self

    def set_creation_rules(self, rules: List[Rule]):
        for rule in rules:
            rule.name = self.name
        self.creation_rules = rules
        return self

    def set_update_rules(self, rules: List[Rule]):
        for rule in rules:
            rule.name = self.name
        self.update_rules = rules
        return self

    def get_rules(self) -> List[Rule]:
        """获取全局验证规则。"""
        return self.rules

    def get_creation_rules(self) -> List[Rule]:
        """获取创建表单验证规则。"""
        return self.creation_rules

    def get_update_rules(self) -> List[Rule]:
        """获取更新表单验证规则。"""
        return self.update_rules

    def set_value_prop_name(self, value_prop_name: str):
        """设置子节点的值的属性。"""
        self.value_prop_name = value_prop_name
        return self

    def set_wrapper_col(self, col: Any):
        """设置输入控件的布局样式。"""
        self.wrapper_col = col
        return self

    def set_column(self, f: Callable[[Column], Column]):
        """设置列表页、详情页中列属性。"""
        self.column = f(self.column)
        return self

    def set_align(self, align: str):
        """设置列的对齐方式。"""
        self.align = align
        return self

    def set_fixed(self, fixed: Any):
        """设置列是否固定。"""
        self.fixed = fixed
        return self

    def set_editable(self, editable: bool):
        """设置表格列是否可编辑。"""
        self.editable = editable
        return self

    def set_ellipsis(self, ellipsis: bool):
        """设置是否自动缩略。"""
        self.ellipsis = ellipsis
        return self

    def set_copyable(self, copyable: bool):
        """设置是否支持复制。"""
        self.copyable = copyable
        return self

    def set_filters(self, filters: Any):
        """设置表头的筛选菜单项。"""
        if isinstance(filters, dict):
            tmp_filters = [{"text": v, "value": k} for k, v in filters.items()]
            self.filters = tmp_filters
        else:
            self.filters = filters
        return self

    def set_order(self, order: int):
        """设置查询表单中的权重。"""
        self.order = order
        return self

    def set_sorter(self, sorter: bool):
        """设置可排序列。"""
        self.sorter = sorter
        return self

    def set_span(self, span: int):
        """设置包含列的数量。"""
        self.span = span
        return self

    def set_column_width(self, width: int):
        """设置列宽。"""
        self.column_width = width
        return self

    def set_ignore(self, ignore: bool):
        """设置是否忽略保存到数据库。"""
        self.ignore = ignore
        return self

    def set_when(self, *value: Any):
        """设置 When 组件数据。"""
        w = When()
        i = Item(body=None)
        operator: str = "="
        option: Any = None

        if len(value) == 2:
            operator = "="
            option = value[0]
            callback = value[1]
            i.body = callback()
        elif len(value) == 3:
            operator = value[0]
            option = value[1]
            callback = value[2]
            i.body = callback()

        get_option = str(option)
        if operator == "!=":
            i.condition = f"<%=String({self.name}) !== '{get_option}' %>"
        elif operator == "=":
            i.condition = f"<%=String({self.name}) === '{get_option}' %>"
        elif operator == ">":
            i.condition = f"<%=String({self.name}) > '{get_option}' %>"
        elif operator == "<":
            i.condition = f"<%=String({self.name}) < '{get_option}' %>"
        elif operator == "<=":
            i.condition = f"<%=String({self.name}) <= '{get_option}' %>"
        elif operator == ">=":
            i.condition = f"<%=String({self.name}) >= '{get_option}' %>"
        elif operator == "has":
            i.condition = f"<%=(String({self.name}).indexOf('{get_option}') !=-1) %>"
        elif operator == "in":
            json_str = json.dumps(option)
            i.condition = f"<%=({json_str}.indexOf({self.name}) !=-1) %>"
        else:
            i.condition = f"<%=String({self.name}) === '{get_option}' %>"

        i.condition_name = self.name
        i.condition_operator = operator
        i.option = option
        self.when_item.append(i)
        self.when = w.set_items(self.when_item)
        return self

    def get_when(self) -> Optional[When]:
        """获取 When 组件数据。"""
        return self.when

    def hide_from_index(self, callback: bool):
        """指定元素是否在列表页隐藏。"""
        self.show_on_index = not callback
        return self

    def hide_from_detail(self, callback: bool):
        """指定元素是否在详情页隐藏。"""
        self.show_on_detail = not callback
        return self

    def hide_when_creating(self, callback: bool):
        """指定元素是否在创建页隐藏。"""
        self.show_on_creation = not callback
        return self

    def hide_when_updating(self, callback: bool):
        """指定元素是否在编辑页隐藏。"""
        self.show_on_update = not callback
        return self

    def hide_when_exporting(self, callback: bool):
        """指定元素是否在导出文件中隐藏。"""
        self.show_on_export = not callback
        return self

    def hide_when_importing(self, callback: bool):
        """指定元素是否在导入文件中隐藏。"""
        self.show_on_import = not callback
        return self

    def on_index_showing(self, callback: bool):
        """指定元素是否在列表页显示。"""
        self.show_on_index = callback
        return self

    def on_detail_showing(self, callback: bool):
        """指定元素是否在详情页显示。"""
        self.show_on_detail = callback
        return self

    def show_on_creating(self, callback: bool):
        """指定元素是否在创建页显示。"""
        self.show_on_creation = callback
        return self

    def show_on_updating(self, callback: bool):
        """指定元素是否在编辑页显示。"""
        self.show_on_update = callback
        return self

    def show_on_exporting(self, callback: bool):
        """指定元素是否在导出文件中显示。"""
        self.show_on_export = callback
        return self

    def show_on_importing(self, callback: bool):
        """指定元素是否在导入文件中显示。"""
        self.show_on_import = callback
        return self

    def only_on_index(self):
        """指定元素只在列表页显示。"""
        self.show_on_index = True
        self.show_on_detail = False
        self.show_on_creation = False
        self.show_on_update = False
        self.show_on_export = False
        self.show_on_import = False
        return self

    def only_on_detail(self):
        """指定元素只在详情页显示。"""
        self.show_on_index = False
        self.show_on_detail = True
        self.show_on_creation = False
        self.show_on_update = False
        self.show_on_export = False
        self.show_on_import = False
        return self

    def only_on_forms(self):
        """指定元素只在表单页显示。"""
        self.show_on_index = False
        self.show_on_detail = False
        self.show_on_creation = True
        self.show_on_update = True
        self.show_on_export = False
        self.show_on_import = False
        return self

    def only_on_creating(self):
        """指定元素只在创建页显示。"""
        self.show_on_index = False
        self.show_on_detail = False
        self.show_on_creation = True
        self.show_on_update = False
        self.show_on_export = False
        self.show_on_import = False
        return self

    def only_on_updating(self):
        """指定元素只在编辑页显示。"""
        self.show_on_index = False
        self.show_on_detail = False
        self.show_on_creation = False
        self.show_on_update = True
        self.show_on_export = False
        self.show_on_import = False
        return self

    def only_on_export(self):
        """指定元素只在导出文件中显示。"""
        self.show_on_index = False
        self.show_on_detail = False
        self.show_on_creation = False
        self.show_on_update = False
        self.show_on_export = True
        self.show_on_import = False
        return self

    def only_on_import(self):
        """指定元素只在导入文件中显示。"""
        self.show_on_index = False
        self.show_on_detail = False
        self.show_on_creation = False
        self.show_on_update = False
        self.show_on_export = False
        self.show_on_import = True
        return self

    def except_on_forms(self):
        """指定元素在表单页隐藏。"""
        self.show_on_index = True
        self.show_on_detail = True
        self.show_on_creation = False
        self.show_on_update = False
        self.show_on_export = True
        self.show_on_import = True
        return self

    def is_shown_on_update(self) -> bool:
        """检查元素是否在编辑页显示。"""
        return self.show_on_update

    def is_shown_on_index(self) -> bool:
        """检查元素是否在列表页显示。"""
        return self.show_on_index

    def is_shown_on_detail(self) -> bool:
        """检查元素是否在详情页显示。"""
        return self.show_on_detail

    def is_shown_on_creation(self) -> bool:
        """检查元素是否在创建页显示。"""
        return self.show_on_creation

    def is_shown_on_export(self) -> bool:
        """检查元素是否在导出文件中显示。"""
        return self.show_on_export

    def is_shown_on_import(self) -> bool:
        """检查元素是否在导入文件中显示。"""
        return self.show_on_import

    def get_value_enum(self) -> Dict[Any, Any]:
        """获取当前列值的枚举。"""
        data: Dict[Any, Any] = {}
        return data

    def set_callback(self, closure: Optional[Callable[[Dict[str, Any]], Any]]):
        """设置回调函数。"""
        if closure:
            self.callback = closure
        return self

    def get_callback(self) -> Optional[Callable[[Dict[str, Any]], Any]]:
        """获取回调函数。"""
        return self.callback

    def set_api(self, api: str):
        """设置获取数据接口。"""
        self.api = api
        return self



from typing import Any, List

from tortoise.queryset import QuerySet

from quark import Message, Request
from quark.template.action import Action


class DeleteRole(Action):

    def __init__(self):
        self.name = "删除"
        self.type = "link"
        self.size = "small"
        self.reload = "table"
        self.set_only_on_index_table_row(True)
        self.with_confirm(
            "确定要删除吗？", "删除后数据将无法恢复，请谨慎操作！", "modal"
        )

    def get_api_params(self) -> List[str]:
        return ["id"]

    async def handle(self, request: Request, query: QuerySet) -> Any:
        id_str = request.query_params.get("id")
        if not id_str:
            return Message.error("参数错误")

        ids = id_str.split(",")
        try:
            ids_int = [int(i) for i in ids]
        except ValueError as e:
            return Message.error(f"参数转换错误: {str(e)}")

        try:
            # 先删除数据库中的记录
            await query.filter(id__in=ids_int).delete()

            # 清理 Casbin 中对应角色的权限
            # casbin_service = CasbinService()
            # for role_id in ids_int:
            #     await casbin_service.remove_role_menu_and_permissions(role_id)

            return Message.success("操作成功")
        except Exception as e:
            return Message.error(str(e))



from pydantic import Field, model_validator
from typing import Any, List, Optional
from ...component import Component

class Item(Component):
    component: str = "whenItem"     # 组件名称
    condition: str = None           # 条件：js表达式语句
    condition_name: str = None      # 需要对比的字段名称
    condition_operator: str = None  # 操作符，= <>
    option: Optional[Any] = None    # 条件符合的属性值
    body: Optional[Any] = None      # 内容

class When(Component):
    component: str = "when"
    items: List[Item] = Field(default_factory=list)

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        return self

    # 设置When组件中需要解析的元素
    def set_items(self, items: List[Item]) -> 'When':
        self.items = items
        return self


from typing import Any, List, Optional

from quark import Message, Request
from pydantic import BaseModel, Field

from ..component.card.card import Card
from ..component.grid.col import Col
from ..component.grid.row import Row


class Dashboard(BaseModel):
    """仪表盘"""

    async def init(self, request: Request):
        """初始化"""
        return self

    async def cards(self, request: Request) -> List[Any]:
        """获取卡片组件列表"""
        return []

    async def render(self, request: Request) -> Any:
        """组件渲染"""
        # 获取卡片组件列表
        cards = await self.cards(request)
        if not cards:
            return "请实现Cards内容"

        cols: List[Col] = []
        body: List[Row] = []
        col_num = 0

        for key, v in enumerate(cards):
            # 断言 statistic 组件类型
            if hasattr(v, "calculate"):
                value = await v.calculate()
                item = Card().set_body(value)
            else:
                item = Card()

            # 获取卡片的列数
            col_span = getattr(v, "col", 0)
            col_info = Col().set_span(col_span).set_body(item)
            cols.append(col_info)
            col_num += col_span

            # 如果列数达到 24，创建一行
            if col_num % 24 == 0:
                row = Row().set_gutter(8).set_body(cols)
                if key != 0:
                    row.set_style({"marginTop": "20px"})
                body.append(row)
                cols = []

        # 如果还有剩余的列，创建一行
        if cols:
            row = Row().set_gutter(8).set_body(cols)
            if col_num > 24:
                row.set_style({"marginTop": "20px"})
            body.append(row)

        return Message.success(
            "ok", body
        )



from typing import Any, Dict, List, Optional

from pydantic import Field, model_validator

from ..component import Component


class Expandable(Component):
    children_column_name: Optional[str] = None
    column_title: Optional[Any] = None
    column_width: Optional[Any] = None
    default_expand_all_rows: bool = None
    default_expanded_row_keys: List[Any] = None
    expanded_row_class_name: Optional[str] = None
    expanded_row_keys: List[Any] = None
    expand_icon: Optional[Any] = None
    expand_row_by_click: bool = None
    fixed: Optional[Any] = None
    indent_size: int = None
    row_expandable: bool = None
    show_expand_column: bool = None


class Table(Component):
    component: str = "table"
    row_key: str = "id"
    api: str = None
    api_type: str = "GET"
    table_layout: str = None
    header_title: str = None
    columns: Any = None
    row_selection: Any = Field(default_factory=list)
    options: Dict[str, bool] = Field(
        default_factory=lambda: {"fullScreen": True, "reload": True, "setting": True}
    )
    search: Any = None
    date_formatter: str = "string"
    column_empty_text: str = "-"
    tool_bar: Any = None
    tree_bar: Any = None
    table_extra_render: Any = None
    expandable: Optional[Expandable] = None
    scroll: Any = None
    striped: bool = False
    datasource: Any = None
    pagination: Any = None
    polling: int = None

    @model_validator(mode="after")
    def init(self):
        self.set_key("table", False)
        return self

    # 设置方法（链式调用）
    def set_style(self, style: dict):
        self.style = style
        return self

    def set_row_key(self, key: str):
        self.row_key = key
        return self

    def set_api(self, api: str):
        self.api = api
        return self

    def set_api_type(self, api_type: str):
        self.api_type = api_type
        return self

    def set_table_layout(self, layout: str):
        self.table_layout = layout
        return self

    def set_title(self, title: str):
        self.header_title = title
        return self

    def set_header_title(self, title: str):
        self.header_title = title
        return self

    def set_searches(self, search: Any):
        self.search = search
        return self

    def set_columns(self, columns: Any):
        self.columns = columns
        return self

    def set_row_selection(self, selection: Any):
        self.row_selection = selection
        return self

    def set_options(self, options: Dict[str, bool]):
        self.options = options
        return self

    def set_date_formatter(self, formatter: str):
        self.date_formatter = formatter
        return self

    def set_column_empty_text(self, text: str):
        self.column_empty_text = text
        return self

    def set_tool_bar(self, toolbar: Any):
        self.tool_bar = toolbar
        return self

    def set_tree_bar(self, treebar: Any):
        self.tree_bar = treebar
        return self

    def set_table_extra_render(self, render: Any):
        self.table_extra_render = render
        return self

    def set_expandable(self, expandable: Expandable):
        self.expandable = expandable
        return self

    def set_scroll(self, scroll: Any):
        self.scroll = scroll
        return self

    def set_striped(self, striped: bool):
        self.striped = striped
        return self

    def set_datasource(self, source: Any):
        self.datasource = source
        return self

    def set_pagination(
        self,
        current: int,
        page_size: int,
        total: int,
        default_current: int,
        page_size_options: list,
    ):
        self.pagination = {
            "current": current,
            "pageSize": page_size,
            "total": total,
            "defaultCurrent": default_current,
            "pageSizeOptions": page_size_options,
        }
        return self

    def set_polling(self, polling: int):
        self.polling = polling
        return self



from pydantic import Field, model_validator
from typing import Any, Dict, Optional
from ..component import Component

class Tabs(Component):
    centered: bool = False
    default_active_key: str = None
    size: str = "default"
    tab_bar_extra_content: Any = None
    tab_bar_gutter: int = 35
    tab_bar_style: Optional[Dict[str, Any]] = Field(default_factory=dict)
    tab_position: str = "top"
    type: str = "line"
    tab_panes: Any = None
    component: str = "tabs"

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        return self
    # 设置方法（链式调用）
    def set_style(self, style: Dict[str, Any]):
        self.style = style
        return self

    def set_centered(self, centered: bool):
        self.centered = centered
        return self

    def set_default_active_key(self, key: str):
        self.default_active_key = key
        return self

    def set_size(self, size: str):
        self.size = size
        return self

    def set_tab_bar_extra_content(self, content: Any):
        self.tab_bar_extra_content = content
        return self

    def set_tab_bar_gutter(self, gutter: int):
        self.tab_bar_gutter = gutter
        return self

    def set_tab_bar_style(self, style: Dict[str, Any]):
        self.tab_bar_style = style
        return self

    def set_tab_position(self, position: str):
        self.tab_position = position
        return self

    def set_type(self, tab_type: str):
        self.type = tab_type
        return self

    def set_tab_panes(self, panes: Any):
        self.tab_panes = panes
        return self


from typing import List, Any
from .fields.id import ID
from .fields.hidden import Hidden
from .fields.text import Text
from .fields.textarea import Textarea
from .fields.password import Password
from .fields.radio import Radio
from .fields.radio import Option as RadioOption
from .fields.image_captcha import ImageCaptcha
from .fields.checkbox import Checkbox
from .fields.checkbox import Option as CheckboxOption
from .fields.date import Date
from .fields.date_range import DateRange
from .fields.datetime import Datetime
from .fields.datetime_range import DatetimeRange
from .fields.switch import Switch
from .fields.tree import Tree
from .fields.icon import Icon
from .fields.select import Select
from .fields.select import Option as SelectOption
from .fields.cascader import Cascader
from .fields.cascader import Option as CascaderOption
from .fields.image import Image
from .fields.file import File
from .fields.display import Display
from .fields.editor import Editor
from .fields.group import Group
from .fields.list import List as ListField
from .fields.map import Map
from .fields.geofence import Geofence
from .fields.month import Month
from .fields.number import Number
from .fields.quarter import Quarter
from .fields.search import Search
from .fields.time_range import TimeRange
from .fields.time import Time
from .fields.week import Week
from .fields.year import Year
from .fields.selects import Selects
from .fields.tree_select import TreeSelect
from .fields.space import Space
from .fields.compact import Compact
from .fields.fieldset import Fieldset
from .fields.dependency import Dependency
from .fields.transfer import Transfer
from .fields.sms_captcha import SmsCaptcha
from .fields.image_picker import ImagePicker
from .fields.action import Action


def field_parser(field: Any, params: List[Any], placeholder: str) -> Any:
    """
    解析字段参数，并设置字段的相应属性。

    :param field: 字段组件
    :param params: 参数列表，可能包括名称、标签、回调等
    :param placeholder: 占位符，默认为空
    :return: 更新后的字段组件
    """
    if len(params) == 1:
        field.name = str(params[0])
    elif len(params) == 2:
        field.name = str(params[0])
        field.label = str(params[1])
    elif len(params) == 3:
        field.name = str(params[0])
        field.label = str(params[1])

        if callable(params[2]):
            field.callback = params[2]

    if placeholder and len(params) > 1:
        field.placeholder = placeholder + str(params[1])

    return field

def id(*params) -> 'ID':
    """生成 ID 字段组件"""
    return field_parser(ID(), params, "")

def hidden(*params) -> 'Hidden':
    """生成隐藏字段组件"""
    return field_parser(Hidden(), params, "")

def text(*params) -> 'Text':
    """生成文本框字段组件"""
    return field_parser(Text(), params, "请输入")

def textarea(*params) -> 'Textarea':
    """生成文本域字段组件"""
    return field_parser(Textarea(), params, "请输入")

def password(*params) -> 'Password':
    """生成密码框字段组件"""
    return field_parser(Password(), params, "请输入")

def radio(*params) -> 'Radio':
    """生成单选框字段组件"""
    return field_parser(Radio(), params, "")

def radio_option(label: str, value: Any) -> 'RadioOption':
    """生成单选框选项"""
    return RadioOption(label=label, value=value)

def checkbox(*params) -> 'Checkbox':
    """生成多选框字段组件"""
    return field_parser(Checkbox(), params, "")

def checkbox_option(label: str, value: Any) -> 'CheckboxOption':
    """生成多选框选项"""
    return CheckboxOption(label=label, value=value)

def date(*params) -> 'Date':
    """生成日期字段组件"""
    return field_parser(Date(), params, "")

def date_range(*params) -> 'DateRange':
    """生成日期范围字段组件"""
    return field_parser(DateRange(), params, "")

def datetime(*params) -> 'Datetime':
    """生成日期时间字段组件"""
    return field_parser(Datetime(), params, "")

def datetime_range(*params) -> 'DatetimeRange':
    """生成日期时间范围字段组件"""
    return field_parser(DatetimeRange(), params, "")

def switch(*params) -> 'Switch':
    """生成开关字段组件"""
    return field_parser(Switch(), params, "")

def tree(*params) -> 'Tree':
    """生成树形选择字段组件"""
    return field_parser(Tree(), params, "请选择")

def icon(*params) -> 'Icon':
    """生成图标选择字段组件"""
    return field_parser(Icon(), params, "请选择")

def select(*params) -> 'Select':
    """生成下拉框字段组件"""
    return field_parser(Select(), params, "请选择")

def select_option(label: str, value: Any) -> 'SelectOption':
    """生成下拉框选项"""
    return SelectOption(label=label, value=value)

def cascader(*params) -> 'Cascader':
    """生成级联选择字段组件"""
    return field_parser(Cascader(), params, "请选择")

def image(*params) -> 'Image':
    """生成图片选择字段组件"""
    return field_parser(Image(), params, "")

def file(*params) -> 'File':
    """生成文件上传字段组件"""
    return field_parser(File(), params, "")

def display(*params: str) -> 'Display':
    """生成文本展示组件"""
    field = Display()
    if len(params) == 1:
        field.set_label(" ").set_value(params[0]).set_colon(False)
    elif len(params) == 2:
        field.set_label(params[0]).set_value(params[1])
    return field

def editor(*params) -> 'Editor':
    """生成富文本编辑器组件"""
    return field_parser(Editor(), params, "")

def group(*options) -> 'Group':
    """生成分组字段组件"""
    field = Group()

    if len(options) == 1:
        field.set_body(options[0])

    if len(options) == 2:
        field.set_title(options[0]).set_body(options[1])

    return field

def list(*params) -> 'ListField':
    """生成列表字段组件"""
    return field_parser(ListField(), params, "")

def map(*params) -> 'Map':
    """生成地图选择字段组件"""
    return field_parser(Map(), params, "")

def geofence(*params) -> 'Geofence':
    """生成地理围栏选择字段组件"""
    return field_parser(Geofence(), params, "")

def month(*params) -> 'Month':
    """生成月份选择字段组件"""
    return field_parser(Month(), params, "")

def number(*params) -> 'Number':
    """生成数字输入框字段组件"""
    return field_parser(Number(), params, "请输入")

def quarter(*params) -> 'Quarter':
    """生成季度选择字段组件"""
    return field_parser(Quarter(), params, "")

def search(*params) -> 'Search':
    """生成搜索字段组件"""
    return field_parser(Search(), params, "")

def time_range(*params) -> 'TimeRange':
    """生成时间范围字段组件"""
    return field_parser(TimeRange(), params, "")

def time(*params) -> 'Time':
    """生成时间选择字段组件"""
    return field_parser(Time(), params, "")

def week(*params) -> 'Week':
    """生成周选择字段组件"""
    return field_parser(Week(), params, "")

def year(*params) -> 'Year':
    """生成年份选择字段组件"""
    return field_parser(Year(), params, "")

def selects(body: Any) -> 'Selects':
    """生成联动选择组件"""
    field = Selects().set_body(body)
    return field

def tree_select(*params) -> 'TreeSelect':
    """生成树形选择联动组件"""
    return field_parser(TreeSelect(), params, "")

def space(*options) -> 'Space':
    """生成间距布局组件"""
    field = Space()

    if len(options) == 1:
        field.set_body(options[0])

    if len(options) == 2:
        field.set_label(options[0]).set_body(options[1])

    return field

def compact(label: str, items: List[Any]) -> 'Compact':
    """生成紧凑布局组件"""
    field = Compact().set_label(label).set_body(items)
    return field

def fieldset(*params) -> 'Fieldset':
    """生成字段集合组件"""
    return field_parser(Fieldset(), params, "")

def dependency() -> 'Dependency':
    """数据联动组件"""
    return Dependency()

def transfer(*params) -> 'Transfer':
    """穿梭框组件"""
    return field_parser(Transfer(), params, "")

def image_captcha(*params) -> 'ImageCaptcha':
    """图形验证码组件"""
    return field_parser(ImageCaptcha(), params, "")

def sms_captcha(*params) -> 'SmsCaptcha':
    """短信验证码组件"""
    return field_parser(SmsCaptcha(), params, "")

def image_picker(*params) -> 'ImagePicker':
    """图片选择器组件"""
    return field_parser(ImagePicker(), params, "")

def action(*params) -> 'Action':
    """行为组件"""
    return field_parser(Action(), params, "")


import json
from typing import Any, List

from tortoise.queryset import QuerySet

from quark import Request

from ...services.attachment import AttachmentService
from ..performs_queries import PerformsQueries
from ..resolves_actions import ResolvesActions


class IndexRequest:

    # 请求对象
    request: Request

    # 查询对象
    query: QuerySet

    # 列表页字段
    fields: list

    # 搜索项
    searches: list

    # 全局数据排序规则
    query_order: List[str]

    # 列表页面的排序规则
    index_query_order: List[str]

    # 分页数量
    page_size: int

    # 分页配置
    page_size_options: list

    def __init__(
        self,
        request: Request,
        resource: Any,
        query: QuerySet,
        query_order: List[str],
        index_query_order: List[str],
        fields: list,
        searches: list,
        page_size: int,
        page_size_options: list,
    ):
        self.request = request
        self.resource = resource
        self.query = query
        self.query_order = query_order
        self.index_query_order = index_query_order
        self.fields = fields
        self.searches = searches
        self.page_size = page_size
        self.page_size_options = page_size_options

    async def query_data(self):
        """
        查询并返回列表数据
        """

        # 获取列过滤条件
        column_filters = self.column_filters()

        # 获取排序规则
        orderings = self.orderings()

        # 构建查询
        query = PerformsQueries(
            request=self.request,
            query=self.query,
        ).build_index_query(self.searches, column_filters)

        # 获取分页配置
        page_size = self.page_size
        page_size_options = self.page_size_options
        if not page_size or not isinstance(page_size, int):
            index_query = PerformsQueries(
                request=self.request,
                query_order=self.query_order,
                index_query_order=self.index_query_order,
            ).apply_index_orderings(query, orderings)

            results = await index_query.all()
            return await self.performs_list(results)

        # 分页参数
        data = self.request.query_params.get("pagination")
        search_params = {}
        if data:
            try:
                search_params = json.loads(data)
            except:
                pass

        current = int(search_params.get("current", 1))
        page_size = int(search_params.get("pageSize", page_size))

        # 获取总数
        total = await query.count()

        # 获取排序规则
        index_query = PerformsQueries(
            request=self.request,
            query_order=self.query_order,
            index_query_order=self.index_query_order,
        ).apply_index_orderings(query, orderings)

        # 获取当前页数据
        results = (
            await index_query.limit(page_size).offset((current - 1) * page_size).all()
        )

        # 解析列表数据
        parsed_items = await self.performs_list(results)

        # 构建返回数据
        data = {
            "current": current,
            "pageSize": page_size,
            "pageSizeOptions": page_size_options,
            "total": total,
            "items": parsed_items,
        }

        return data

    def column_filters(self):
        """
        获取列过滤条件
        """
        filter_str = self.request.query_params.get("filter")
        column_filters = {}
        try:
            column_filters = json.loads(filter_str)
        except:
            pass
        return column_filters

    def orderings(self):
        """
        获取排序规则
        """
        sorter_str = self.request.query_params.get("sorter")
        orderings = {}
        try:
            orderings = json.loads(sorter_str)
        except:
            pass
        return orderings

    async def performs_list(self, items):
        """
        处理列表字段
        """
        result = []
        index_fields = self.fields
        for item in items:
            fields = {}

            for field in index_fields:
                component = field.component
                name = field.name

                if component == "actionField":
                    items_callback = field.callback
                    if items_callback:
                        action_items = await items_callback(item)
                    else:
                        action_items = field.items

                    rendered_actions = []
                    for action in action_items:
                        rendered_actions.append(
                            await ResolvesActions(self.request).build_action(action)
                        )

                    fields[name] = rendered_actions
                else:
                    callback = field.callback
                    if callback:
                        fields[name] = await callback(item)
                    else:
                        value = getattr(item, name, None)
                        if value is None:
                            continue

                        # JSON 字符串解析
                        if isinstance(value, str):
                            if value.startswith("[") or value.startswith("{"):
                                try:
                                    value = json.loads(value)
                                except:
                                    pass

                        # 单独解析时间和日期组件
                        if component in ["datetimeField", "dateField"]:
                            format_str = field.format
                            format_str = format_str.replace("YYYY", "%Y")
                            format_str = format_str.replace("MM", "%m")
                            format_str = format_str.replace("DD", "%d")
                            format_str = format_str.replace("HH", "%H")
                            format_str = format_str.replace("mm", "%M")
                            format_str = format_str.replace("ss", "%S")

                            value = value.strftime(format_str)

                        # 图片字段处理
                        if component in ["imageField", "imagePickerField"]:
                            value = await AttachmentService().get_image_url(value)

                        fields[name] = value

            result.append(fields)

        return await self.resource.before_index_showing(self.request, result)



from tortoise import fields
from tortoise.models import Model


class RoleMenu(Model):
    id = fields.BigIntField(pk=True)
    role_id = fields.BigIntField()
    menu_id = fields.BigIntField()
    guard_name = fields.CharField(max_length=200, default="admin")
    created_at = fields.DatetimeField(null=True)
    updated_at = fields.DatetimeField(auto_now=True, null=True)

    class Meta(Model.Meta):
        table = "role_menus"



from typing import Any, Dict, List

from tortoise.queryset import QuerySet

from quark import Message, Request
from quark.component.form import field
from quark.component.form.fields.select import Option
from quark.component.form.rule import Rule
from quark.services.department import DepartmentService
from quark.services.role import RoleService
from quark.template.action import ModalForm


class DataScope(ModalForm):

    def __init__(self, name="数据权限"):
        self.name = name
        self.type = "link"
        self.size = "small"
        self.destroy_on_close = True
        self.set_only_on_index_table_row(True)
        self.set_api_params(["id", "name"])

    async def fields(self, request: Request) -> List[Any]:
        departments = DepartmentService().get_list()

        return [
            field.hidden("id", "ID"),
            field.text("name", "名称").set_disabled(True),
            field.select("data_scope", "数据范围")
            .set_options(
                [
                    Option(label="全部数据权限", value=1),
                    Option(label="自定数据权限", value=2),
                    Option(label="本部门数据权限", value=3),
                    Option(label="本部门及以下数据权限", value=4),
                    Option(label="仅本人数据权限", value=5),
                ]
            )
            .set_rules([Rule.required("请选择数据范围")])
            .set_default_value(1),
            field.dependency().set_when(
                "data_scope",
                2,
                lambda: [
                    field.tree("department_ids", "数据权限")
                    .set_default_expand_all(True)
                    .set_tree_data(departments, pid="pid", id="id", name="name")
                ],
            ),
        ]

    async def data(self, request: Request) -> Dict[str, Any]:
        id_str = request.query_params.get("id")
        if not id_str:
            return {}

        try:
            role_id = int(id_str)
        except ValueError:
            return {}

        role = RoleService().get_info_by_id(role_id)
        if not role:
            return {}

        # dept_ids = CasbinService().get_role_department_ids(role_id)
        dept_ids = []

        return {
            "id": role.id,
            "name": role.name,
            "data_scope": role.data_scope,
            "department_ids": dept_ids,
        }

    async def handle(self, request: Request, query: QuerySet):
        form = await request.body()

        try:
            RoleService().update_role_data_scope(
                form["id"], form["data_scope"], form.get("department_ids", [])
            )
            return Message.success("操作成功")
        except Exception as e:
            return Message.error(str(e))



import warnings
from typing import Any, Iterable

from tortoise import Tortoise


async def init(
    config: dict | None = None,
    config_file: str | None = None,
    db_url: str | None = None,
    modules: dict[str, Iterable[Any]] | None = None,
) -> None:
    """初始化数据库配置"""
    await Tortoise.init(
        config=config,
        config_file=config_file,
        db_url=db_url,
        modules=modules,
    )

    # 忽略表已存在的警告
    warnings.filterwarnings(
        "ignore", category=Warning, message=".*Table.*already exists.*"
    )
    await Tortoise.generate_schemas()



from .action_log import ActionLogService
from .attachment import AttachmentService
from .attachment_category import AttachmentCategoryService
from .auth import AuthService
from .config import ConfigService
from .department import DepartmentService
from .menu import MenuService
from .permission import PermissionService
from .position import PositionService
from .role import RoleService
from .user import UserService



from typing import Any
from .base import Base


class Action(Base):

    component: str = "actionField"
    """
    组件名称
    """

    def set_items(self, items: Any):
        """设置行为项。"""
        self.items = items
        return self



from typing import Any, Optional
from .base import Base


class ID(Base):

    component: str = "idField"
    """
    组件名称
    """

    default_value: Optional[Any] = None
    """
    默认的选中项
    """

    disabled: bool = False
    """
    整组失效，默认 False
    """

    value: Optional[Any] = None
    """
     指定选中项
    """

    on_index_displayed: bool = True
    """
    在列表页显示字段，默认 True
    """

    on_detail_displayed: bool = True
    """
    在详情页显示字段，默认 True
    """

    on_form_displayed: bool = True
    """
    在表单页显示控件，默认 True
    """

    on_export_displayed: bool = True
    """
    在导出页显示字段，默认 True
    """

    def init(self):
        """
        初始化组件属性。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.component = "idField"
        self.colon = True
        self.label_align = "right"
        self.show_on_index = True
        self.show_on_detail = True
        self.show_on_creation = True
        self.show_on_update = True
        self.show_on_export = True
        self.show_on_import = True
        self.on_index_displayed = True
        self.show_on_import = False

        return self

    def set_on_index_displayed(self, displayed: bool):
        """
        在列表页是否显示字段。

        Args:
            displayed (bool): 是否显示。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.on_index_displayed = displayed
        return self

    def set_on_detail_displayed(self, displayed: bool):
        """
        在详情页是否显示字段。

        Args:
            displayed (bool): 是否显示。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.on_detail_displayed = displayed
        return self

    def set_on_form_displayed(self, displayed: bool):
        """
        在表单页是否显示控件。

        Args:
            displayed (bool): 是否显示。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.on_form_displayed = displayed
        return self

    def set_on_export_displayed(self, displayed: bool):
        """
        在导出页是否显示字段。

        Args:
            displayed (bool): 是否显示。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.on_export_displayed = displayed
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self



from tortoise import fields
from tortoise.models import Model


class AttachmentCategory(Model):
    id = fields.IntField(pk=True)
    source = fields.CharField(max_length=100, null=True)
    uid = fields.IntField(default=0)
    title = fields.CharField(max_length=255)
    sort = fields.IntField(default=0)
    description = fields.CharField(max_length=255, null=True)

    class Meta(Model.Meta):
        table = "attachment_categories"



from typing import Any, Dict, List

from quark import Request

from ...component.menu.menu import Menu
from ..resolves_actions import ResolvesActions
from .action import Action


class Dropdown(Action):
    """
    表示一个下拉菜单组件，支持配置触发方式、位置、样式等。
    """

    # 是否显示箭头图标，默认不显示
    arrow: bool = False

    # 下拉菜单类型
    action_type: str = "dropdown"

    # 菜单弹出位置，默认为 bottomLeft
    placement: str = "bottomLeft"

    # 触发下拉的行为，默认为 ["hover"]
    trigger: List[str] = ["hover"]

    # 下拉根元素的样式，默认为空字典
    overlay_style: Dict[str, Any] = None

    # 下拉菜单行为列表
    actions: List[Any] = None

    def get_arrow(self) -> bool:
        """获取是否显示箭头图标"""
        return self.arrow

    def get_placement(self) -> str:
        """获取菜单弹出位置：bottomLeft, bottomCenter, bottomRight 等"""
        return self.placement

    def get_trigger(self) -> List[str]:
        """获取触发下拉的行为，移动端不支持 hover"""
        return self.trigger

    def get_overlay_style(self) -> Dict[str, Any]:
        """获取下拉根元素的样式"""
        return self.overlay_style

    async def get_menu(self, request: Request) -> Menu:
        """
        构建下拉菜单内容
        """
        items = []

        # 获取行为
        actions = self.get_actions()

        # 解析行为并构建菜单项
        for action in actions:
            items.append(await ResolvesActions(request).build_action(action))

        return Menu().set_items(items)

    def set_actions(self, actions: List[Any]) -> Any:
        """
        设置下拉菜单行为
        """
        self.actions = actions
        return self

    def get_actions(self) -> List[Any]:
        """
        获取下拉菜单行为
        """
        return self.actions



from typing import List

from quark.models.menu_permission import MenuPermission
from quark.models.role_permission import RolePermission

from ..component.form.fields.transfer import DataSource
from ..models.permission import Permission


class PermissionService:
    def __init__(self):
        pass

    async def list(self):
        permissions = await Permission.all()
        list_options = [
            {"label": permission.name, "value": permission.id}
            for permission in permissions
        ]
        return list_options

    async def data_source(self):
        permissions = await Permission.all()
        data_source = [
            DataSource(
                key=permission.id,
                title=permission.name,
                description=permission.remark,
            )
            for permission in permissions
        ]
        return data_source

    async def get_list_by_ids(self, permission_ids):
        permissions = await Permission.filter(id__in=permission_ids).all()
        return permissions

    async def get_list_by_names(self, permission_names):
        permissions = await Permission.filter(name__in=permission_names).all()
        return permissions

    async def get_menu_permissions(self, menu_id: int):
        permissions = await MenuPermission.filter(menu_id=menu_id).all()
        return permissions

    async def get_permissions_by_role_ids(self, role_ids: List[int]):
        permissions = await RolePermission.filter(role_id__in=role_ids).all()
        return permissions

    async def add_menu_permission(self, menu_id: int, permission_ids: List[int]):
        await MenuPermission.filter(menu_id=menu_id).delete()
        await MenuPermission.bulk_create(
            [
                MenuPermission(menu_id=menu_id, permission_id=permission_id)
                for permission_id in permission_ids
            ]
        )



from tortoise import fields
from tortoise.models import Model


class Role(Model):
    id = fields.IntField(pk=True)
    name = fields.CharField(max_length=255)
    data_scope = fields.IntField(default=1)  # 1：全部权限 ...
    guard_name = fields.CharField(max_length=100)
    status = fields.IntField(default=1)
    created_at = fields.DatetimeField(auto_now_add=True)
    updated_at = fields.DatetimeField(auto_now=True)

    class Meta(Model.Meta):
        table = "roles"

    @classmethod
    async def seeder(cls):
        """
        初始化角色数据：普通角色
        """
        if not await cls.filter(name="普通角色").exists():
            await cls.create(name="普通角色", guard_name="admin", data_scope=1)



from typing import List

from tortoise.queryset import QuerySet

from quark import Message, Request
from quark.template.action import Action


class BatchEnable(Action):

    def __init__(self, name: str = "批量启用"):
        self.name = name
        self.type = "default"
        self.size = "small"
        self.reload = "table"
        self.batch = True
        self.with_confirm("确定要启用吗？", "启用后数据将正常使用！", "modal")
        self.set_only_on_index(True)

    def get_api_params(self) -> List[str]:
        return ["id"]

    async def handle(self, request: Request, query: QuerySet):
        id_param = request.query_params.get("id")
        if not id_param:
            return Message.error("参数错误")

        try:
            ids = [int(i) for i in id_param.split(",")]
            await query.filter(id__in=ids).update(status=1)
            return Message.success("操作成功")
        except Exception as e:
            return Message.error(str(e))



from .search import Search


class Radio(Search):
    """单选组件"""

    def __init__(self, column: str = "", name: str = ""):
        self.component = "radioField"
        return self

    # 设置 Option
    def option(self, label: str, value):
        return {"value": value, "label": label}

    # 设置属性，示例：[{"value": 1, "label": "男"}, {"value": 2, "label": "女"}]
    #
    # 或者
    #
    # set_options(options, "label_name", "value_name")
    def set_options(self, *args):
        if len(args) == 1:
            if isinstance(args[0], list):
                self.radio_options = args[0]
                return self

        if len(args) == 3:
            # 假设 radio.list_to_options 实现了字段映射转换逻辑
            self.radio_options = self.list_to_options(args[0], args[1], args[2])

        return self

    @staticmethod
    def list_to_options(data, label_key, value_key):
        """模拟从列表结构生成选项"""
        return [{"label": item[label_key], "value": item[value_key]} for item in data]



from datetime import datetime, timedelta
from typing import Optional

from fastapi import HTTPException, Request
from jose import JWTError, jwt
from tortoise.exceptions import DoesNotExist

from quark.models.role_permission import RolePermission
from quark.models.user_role import UserRole
from quark.services.permission import PermissionService
from quark.services.role import RoleService
from quark.schemas import UserInfoResponse

from .. import config
from ..models.user import User
from ..services.user import UserService
from ..utils import verify_password


class AuthService:

    def __init__(self, request: Request):
        self.request = request

    def create_token(
        self, data: dict, expires_delta: Optional[timedelta] = None
    ) -> str:
        to_encode = data.copy()
        expire = datetime.now() + (
            expires_delta or timedelta(seconds=24 * 60 * 60 * 30)
        )
        to_encode.update({"exp": expire})
        return jwt.encode(to_encode, config.get("APP_SECRET_KEY"))

    def get_token(self) -> str:
        auth_header = self.request.headers.get("Authorization")
        if auth_header is None:
            raise HTTPException(status_code=401, detail="Authorization header missing")

        try:
            scheme, token = auth_header.split()
            if scheme.lower() != "bearer":
                raise HTTPException(
                    status_code=401, detail="Invalid authentication scheme"
                )
        except ValueError:
            raise HTTPException(
                status_code=401, detail="Invalid authorization header format"
            )

        return token

    async def login(
        self, username: str, password: str, guard_name: str = "user"
    ) -> str:
        user = await UserService().get_info_by_username(username)
        if not user or not verify_password(password, user.password):
            raise HTTPException(status_code=401, detail="用户名或密码错误")

        # 更新最后登录
        await UserService().update_last_login(
            user.id, self.get_real_ip(self.request), datetime.now()
        )

        # 构造 JWT payload
        claims = {
            "id": user.id,
            "username": user.username,
            "nickname": user.nickname,
            "sex": user.sex,
            "email": user.email,
            "phone": user.phone,
            "avatar": user.avatar,
            "guard_name": guard_name,
            "iss": "QuarkCloud",
            "sub": "UserToken",
        }
        return self.create_token(claims)

    # 验证并返回当前用户
    async def get_current_user(self, guard_name: str = "user") -> User:
        credentials_exception = HTTPException(
            status_code=401,
            detail="Could not validate credentials",
            headers={"WWW-Authenticate": "Bearer"},
        )
        try:
            payload = jwt.decode(self.get_token(), config.get("APP_SECRET_KEY"))
            user_id: int = payload.get("id") or 0
            token_guard: str = payload.get("guard_name") or ""
            if user_id is None or token_guard != guard_name:
                raise credentials_exception
        except JWTError:
            raise credentials_exception

        try:
            user = await User.get(id=user_id)
        except DoesNotExist:
            raise HTTPException(status_code=404, detail="User not found")

        if user.status != 1:
            raise HTTPException(status_code=403, detail="User is disabled")
        return user

    async def get_current_admin(self) -> UserInfoResponse:
        userInfo = await self.get_current_user("admin")
        getuser = UserInfoResponse(
            id=userInfo.id,
            username=userInfo.username,
            nickname=userInfo.nickname,
            email=userInfo.email,
            phone=userInfo.phone,
            avatar=userInfo.avatar,
            buttons=[],
            roles=["R_SUPER"]
        )
        return getuser

    async def check_permission(self, path: str, method: str) -> bool:
        admin_info = await self.get_current_admin()
        role_ids = await RoleService().get_role_ids_by_user_id(admin_info.id)

        if not role_ids:
            return False

        placeholders = ",".join(str(role_id) for role_id in role_ids)
        rows = await RolePermission.raw(
            f"""
            SELECT p.id, p.name, p.guard_name, p.path, p.method
            FROM role_has_permissions rhp
            JOIN permissions p ON rhp.permission_id = p.id
            WHERE rhp.role_id IN ({placeholders}) AND p.guard_name = 'admin' AND p.path = '{path}' AND p.method = 'Any'
            """
        )
        if rows:
            return True

        rows = await RolePermission.raw(
            f"""
            SELECT p.id, p.name, p.guard_name, p.path, p.method
            FROM role_has_permissions rhp
            JOIN permissions p ON rhp.permission_id = p.id
            WHERE rhp.role_id IN ({placeholders}) AND p.guard_name = 'admin' AND p.path = '{path}' AND p.method = '{method}'
            """
        )

        if rows:
            return True
        return False

    def get_real_ip(self, request: Request):
        forwarded_for = request.headers.get("X-Forwarded-For")
        if forwarded_for:
            ip = forwarded_for.split(",")[0]
        else:
            ip = request.client.host if request.client else ""
        return ip



import json
from typing import Any, Dict, List

from tortoise.expressions import Q
from tortoise.queryset import QuerySet

from quark import Request, Resource, models, services, utils
from quark.app import actions, searches
from quark.component.form import Rule, field


class User(Resource):
    """
    用户管理
    """

    async def init(self, request: Request) -> Any:

        # 部门列表
        departments = await services.DepartmentService().get_list()

        # 部门树
        self.table_tree_bar.set_name("departmentIds").set_tree_data(
            departments, "pid", "id", "name"
        )

        # 页面标题
        self.title = "用户"

        # 模型
        self.model = models.User

        return self

    async def index_query(self, request: Request) -> QuerySet:
        """
        列表查询
        """
        query = await self.query(request)

        # 从请求中获取 departmentIds 参数
        department_ids_param = request.query_params.get("departmentIds")
        if not department_ids_param:
            return query

        try:
            ids: List[int] = json.loads(department_ids_param)
        except json.JSONDecodeError:
            return query

        if not ids:
            return query

        # 拓展 ids 为包含所有子部门 ID
        all_ids = ids.copy()
        for dep_id in ids:
            children_ids = await services.DepartmentService().get_children_ids(dep_id)
            all_ids.extend(children_ids)

        # 去重
        all_ids = list(set(all_ids))

        # 构建筛选条件
        return query.filter(Q(department_id__in=all_ids))

    async def fields(self, request: Request) -> List[Any]:
        """字段定义"""

        # 获取所有角色
        roles = await services.RoleService().get_list()

        # 获取所有部门
        departments = await services.DepartmentService().get_list()

        # 获取所有职位
        positions = await services.PositionService().get_list()

        return [
            field.id("id", "ID"),
            field.image("avatar", "头像"),
            field.text("username", "用户名")
            .set_rules(
                [
                    Rule.required("用户名必须填写"),
                    Rule.min(6, "用户名不能少于6个字符"),
                    Rule.max(20, "用户名不能超过20个字符"),
                ]
            )
            .set_creation_rules(
                [
                    Rule.unique("users", "username", "用户名已存在"),
                ]
            )
            .set_update_rules(
                [
                    Rule.unique("users", "username", "{id}", "用户名已存在"),
                ]
            ),
            field.text("nickname", "昵称")
            .set_editable(True)
            .set_rules(
                [
                    Rule.required("昵称必须填写"),
                ]
            ),
            field.checkbox("role_ids", "角色").set_options(roles).only_on_forms(),
            field.tree_select("department_id", "部门")
            .set_tree_data(departments, "pid", "name", "id")
            .only_on_forms(),
            field.checkbox("position_ids", "职位")
            .set_options(positions)
            .only_on_forms(),
            field.text("email", "邮箱")
            .set_editable(True)
            .set_rules(
                [
                    Rule.required("邮箱必须填写"),
                ]
            )
            .only_on_forms(),
            field.text("phone", "手机号")
            .set_rules(
                [
                    Rule.required("手机号必须填写"),
                ]
            )
            .set_creation_rules(
                [
                    Rule.unique("users", "phone", "手机号已存在"),
                ]
            )
            .set_update_rules(
                [
                    Rule.unique("users", "phone", "{id}", "手机号已存在"),
                ]
            ),
            field.radio("sex", "性别")
            .set_options([field.radio_option("男", 1), field.radio_option("女", 2)])
            .set_filters(True)
            .set_default_value(1),
            field.password("password", "密码")
            .set_rules(
                [
                    Rule.regexp(r"/^.{6,}$/", "密码不少于六位"),
                    Rule.regexp(r"/.*[A-Z].*/", "至少包含一个大写字母"),
                    Rule.regexp(r"/.*[a-z].*/", "至少包含一个小写字母"),
                    Rule.regexp(r"/.*[0-9].*/", "至少包含一个数字"),
                    Rule.regexp(
                        r"/.*[!@#$%^&*()_+{}\[\]:;<>,.?~\\/-].*/",
                        "至少包含一个特殊字符",
                    ),
                ]
            )
            .set_creation_rules(
                [
                    Rule.required("密码必须填写"),
                ]
            )
            .set_help(
                "密码不少于六位，且至少包含一个大写字母、小写字母、数字和特殊字符"
            )
            .only_on_forms(),
            field.datetime("last_login_time", "最后登录时间").only_on_index(),
            field.switch("status", "状态")
            .set_editable(True)
            .set_true_value("正常")
            .set_false_value("禁用")
            .set_default_value(True),
        ]

    async def searches(self, request: Request) -> List[Any]:
        """搜索项定义"""
        return [
            searches.Input("username", "用户名"),
            searches.Input("nickname", "昵称"),
            searches.Status(),
            searches.DatetimeRange("last_login_time", "登录时间"),
        ]

    async def actions(self, request: Request) -> List[Any]:
        """行为定义"""
        return [
            actions.CreateDrawer(self),
            actions.BatchDelete(),
            actions.EditDrawer(self),
            actions.DeleteSpecial(),
            actions.FormSubmit(),
            actions.FormReset(),
            actions.FormBack(),
            actions.FormExtraBack(),
        ]

    async def before_editing(
        self, request: Request, data: Dict[str, Any]
    ) -> Dict[str, Any]:
        """
        编辑页面显示前回调
        """

        # 密码不显示
        del data["password"]

        # 获取用户角色
        data["role_ids"] = await services.RoleService().get_role_ids_by_user_id(
            data["id"]
        )

        # 返回数据
        return data

    async def before_saving(self, request, submit_data):

        # 密码处理
        if submit_data.get("password"):
            submit_data["password"] = utils.hash_password(submit_data["password"])

        return submit_data

    async def after_saved(self, request, id, data, result):
        await services.RoleService().save_roles_by_user_id(id, data["role_ids"])



from typing import Any, Dict, List

from quark import Request
from quark.services.auth import AuthService
from quark.template.action import Action


class Import(Action):

    def __init__(self, name: str = "导入数据"):
        self.name = name
        self.destroy_on_close = True
        self.set_only_on_index(True)

    async def get_body(self, request: Request) -> Dict[str, Any]:
        resource = request.url.path.get("resource", "")
        token = AuthService(request).get_token()

        api = f"/api/admin/{resource}/import"
        template_link = f"/api/admin/{resource}/import/template?token={token}"
        get_tpl = {
            "type": "tpl",
            "body": f"模板文件: <a href='{template_link}' target='_blank'>下载模板</a>",
            "style": {"marginLeft": "50px"},
        }

        fields = [
            {
                "type": "space",
                "body": [get_tpl],
                "direction": "vertical",
                "size": "middle",
                "style": {"marginBottom": "20px"},
            },
            {
                "type": "file",
                "name": "fileId",
                "label": "导入文件",
                "limitNum": 1,
                "limitType": [
                    "application/vnd.ms-excel",
                    "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
                ],
                "help": "请上传xls格式的文件",
            },
        ]

        form_component = {
            "type": "form",
            "key": "importModalForm",
            "api": api,
            "body": fields,
            "labelCol": {"span": 6},
            "wrapperCol": {"span": 18},
            "style": {"paddingTop": "24px"},
        }

        return form_component

    async def get_actions(self, request: Request) -> List[Dict[str, Any]]:
        return [
            {
                "type": "action",
                "label": "取消",
                "actionType": "cancel",
            },
            {
                "type": "action",
                "label": "提交",
                "withLoading": True,
                "reload": "table",
                "actionType": "submit",
                "btnType": "primary",
                "submitForm": "importModalForm",
            },
        ]



from typing import Dict, Optional, Any
from .base import Base


class Captcha(Base):
    """
    表示短信验证码按钮属性的类。

    Attributes:
        size (Optional[str]): 验证码按钮大小。
        text (Optional[str]): 验证码按钮文案。
        url (Optional[str]): 获取验证码请求地址。
    """

    size: Optional[str] = None
    text: Optional[str] = None
    url: Optional[str] = None


class SmsCaptcha(Base):

    component: str = "smsCaptchaField"
    """
    组件名称
    """

    addon_after: Optional[Any] = None
    """
    带标签的 input，设置后置标签，默认值为 None
    """

    addon_before: Optional[Any] = None
    """
    带标签的 input，设置前置标签，默认值为 None
    """

    allow_clear: bool = False
    """
    可以点击清除图标删除内容，默认值为 False
    """

    bordered: bool = True
    """
    是否有边框，默认值为 True
    """

    default_value: Optional[Any] = None
    """
    默认的选中项，默认值为 None
    """

    disabled: Optional[Any] = None
    """
    禁用，默认值为 None
    """

    id: Optional[str] = None
    """
    输入框的 id，默认值为 None
    """

    max_length: int = 200
    """
    最大长度，默认值为 200
    """

    show_count: bool = False
    """
    是否展示字数，默认值为 False
    """

    status: Optional[str] = None
    """
    设置校验状态,'error' | 'warning'，默认值为 None
    """

    prefix: Optional[Any] = None
    """
    带有前缀图标的 input，默认值为 None
    """

    size: Optional[str] = None
    """
    控件大小。注：标准表单内的输入框大小限制为 middle，可选 large | middle | small，默认值为 None
    """

    suffix: Optional[Any] = None
    """
    带有后缀图标的 input，默认值为 None
    """

    type: Optional[str] = None
    """
    声明 input 类型，同原生 input 标签的 type 属性，默认值为 None
    """

    value: Optional[Any] = None
    """
    指定选中项,string[] | number[]，默认值为 None
    """

    placeholder: str = "请输入"
    """
    占位符，默认值为 "请输入"
    """

    style: Optional[Dict[str, Any]] = None
    """
    自定义样式，默认值为 None
    """

    captcha_id_url: Optional[str] = None
    """
    获取验证码ID的URL，默认值为 None
    """

    captcha_url: Optional[str] = None
    """
    获取验证码URL，默认值为 None
    """

    dependency: str = "phone"
    """
    依赖的字段名称，默认为 "phone"
    """

    captcha_props: Optional[Captcha] = None
    """
    验证码属性，默认值为 None
    """

    def set_addon_after(self, addon_after: Any):
        """
        设置带标签的 input 的后置标签。

        Args:
            addon_after (Any): 后置标签。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.addon_after = addon_after
        return self

    def set_addon_before(self, addon_before: Any):
        """
        设置带标签的 input 的前置标签。

        Args:
            addon_before (Any): 前置标签。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.addon_before = addon_before
        return self

    def set_allow_clear(self, allow_clear: bool):
        """
        设置是否可以点击清除图标删除内容。

        Args:
            allow_clear (bool): 是否允许清除。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.allow_clear = allow_clear
        return self

    def set_bordered(self, bordered: bool):
        """
        设置是否有边框。

        Args:
            bordered (bool): 是否有边框。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.bordered = bordered
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self

    def set_id(self, id_str: str):
        """
        设置输入框的 id。

        Args:
            id_str (str): 输入框的 id。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.id = id_str
        return self

    def set_max_length(self, max_length: int):
        """
        设置最大长度。

        Args:
            max_length (int): 最大长度。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.max_length = max_length
        return self

    def set_show_count(self, show_count: bool):
        """
        设置是否展示字数。

        Args:
            show_count (bool): 是否展示字数。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.show_count = show_count
        return self

    def set_status(self, status: str):
        """
        设置校验状态。

        Args:
            status (str): 校验状态，'error' | 'warning'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.status = status
        return self

    def set_placeholder(self, placeholder: str):
        """
        设置输入框占位文本。

        Args:
            placeholder (str): 占位文本。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.placeholder = placeholder
        return self

    def set_prefix(self, prefix: Any):
        """
        设置带有前缀图标的 input。

        Args:
            prefix (Any): 前缀图标。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.prefix = prefix
        return self

    def set_size(self, size: str):
        """
        设置控件大小。

        Args:
            size (str): 控件大小，可选 large | middle | small。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.size = size
        return self

    def set_suffix(self, suffix: Any):
        """
        设置带有后缀图标的 input。

        Args:
            suffix (Any): 后缀图标。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.suffix = suffix
        return self

    def set_type(self, type_str: str):
        """
        设置 input 类型。

        Args:
            type_str (str): input 类型，同原生 input 标签的 type 属性。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.type = type_str
        return self

    def set_captcha_id_url(self, captcha_id_url: str):
        """
        设置获取验证码ID的URL。

        Args:
            captcha_id_url (str): 获取验证码ID的URL。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.captcha_id_url = captcha_id_url
        return self

    def set_captcha_url(self, captcha_url: str):
        """
        设置获取验证码URL。

        Args:
            captcha_url (str): 获取验证码URL。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.captcha_url = captcha_url
        return self

    def set_dependency(self, dependency: str):
        """
        设置依赖的字段名称。

        Args:
            dependency (str): 依赖的字段名称。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.dependency = dependency
        return self

    def set_captcha_props(self, captcha_props: Captcha):
        """
        设置验证码属性。

        Args:
            captcha_props (Captcha): 验证码属性对象。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.captcha_props = captcha_props
        return self



from typing import Any, Optional
from .base import Base


class Hidden(Base):

    component: str = "hiddenField"
    """
    组件名称
    """

    default_value: Optional[Any] = None
    """
    默认的选中项。
    """

    value: Optional[Any] = None
    """
    指定选中项
    """

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self



import json
from bcrypt import hashpw, gensalt, checkpw
from fastapi import Request


def hash_password(password: str) -> str:
    """加密密码"""
    return hashpw(password.encode("utf-8"), gensalt()).decode("utf-8")


def verify_password(password: str, hashed: str) -> bool:
    """验证密码"""
    return checkpw(password.encode("utf-8"), hashed.encode("utf-8"))


def list_to_tree(data, pk: str, pid: str, child: str, root: int):
    """
    将列表数据转换为树形结构
    :param data: 列表数据
    :param pk: 主键字段名（如 "id"）
    :param pid: 父级字段名（如 "parent_id"）
    :param child: 子级字段名（如 "children"）
    :param root: 根节点的父级值（通常为0）
    :return: 树形结构列表
    """
    result = json.loads(json.dumps(data))  # 模拟深拷贝，处理嵌套结构
    tree_list = []

    for item in result:
        if item.get(pid) == root:
            children = list_to_tree(data, pk, pid, child, item.get(pk))
            if children:
                item[child] = children
            else:
                item[child] = None
            tree_list.append(item)

    return tree_list


def tree_to_ordered_list(tree, level: int, field: str, child: str):
    """
    将树结构转换为有序列表，并添加层级前缀
    :param tree: 树结构数据
    :param level: 当前层级
    :param field: 需要添加前缀的字段名
    :param child: 子级字段名
    :return: 有序列表
    """
    ordered_list = []

    for node in tree:
        node[field] = "—" * level + node.get(field, "")
        ordered_list.append(node)
        if node.get(child):
            children = tree_to_ordered_list(node[child], level + 1, field, child)
            ordered_list.extend(children)

    return ordered_list


def is_index(request: Request) -> bool:
    """
    判断当前页面是否为列表页面
    """
    uri_parts = request.url.path.strip("/").split("/")
    return uri_parts[-1] == "index"


def is_creating(request: Request) -> bool:
    """
    判断当前页面是否为创建页面
    """
    uri_parts = request.url.path.strip("/").split("/")
    return uri_parts[-1] in ("create", "store")


def is_editing(request: Request) -> bool:
    """
    判断当前页面是否为编辑页面
    """
    uri_parts = request.url.path.strip("/").split("/")
    return uri_parts[-1] in ("edit", "save")


def is_detail(request: Request) -> bool:
    """
    判断当前页面是否为详情页面
    """
    uri_parts = request.url.path.strip("/").split("/")
    return uri_parts[-1] == "detail"


def is_export(request: Request) -> bool:
    """
    判断当前页面是否为导出页面
    """
    uri_parts = request.url.path.strip("/").split("/")
    return uri_parts[-1] == "export"


def is_import(request: Request) -> bool:
    """
    判断当前页面是否为导入页面
    """
    uri_parts = request.url.path.strip("/").split("/")
    return uri_parts[-1] == "import"


def replace_last(text: str, old: str, new: str) -> str:
    """
    替换字符串中的最后一个匹配项
    """
    index = text.rfind(old)
    if index == -1:
        return text
    return text[:index] + new + text[index + len(old) :]



import asyncio
from typing import Dict

from ..models.config import Config


class ConfigService:
    def __init__(self):
        self.web_config: Dict[str, str] = {}
        self._lock = asyncio.Lock()

    async def refresh(self):
        configs = await Config.filter(status=1).all()
        async with self._lock:
            self.web_config = {config.name: config.value for config in configs}

    async def set_value(self, key: str, value: str):
        config = await Config.get_or_none(name=key)
        if config:
            config.value = value
            await config.save()
        else:
            await Config.create(name=key, value=value, status=1)
        await self.refresh()

    async def get_value(self, key: str) -> str:
        if not self.web_config:
            await self.refresh()
        return self.web_config.get(key, "")



from typing import Any, List

from quark import Dashboard, Request
from quark.app.metrics.system_info import SystemInfo
from quark.app.metrics.team_info import TeamInfo
from quark.app.metrics.total_admin import TotalAdmin
from quark.app.metrics.total_file import TotalFile
from quark.app.metrics.total_image import TotalImage
from quark.app.metrics.total_log import TotalLog


class Index(Dashboard):
    """仪表盘"""

    async def init(self, request: Request):
        return self

    async def cards(self, request: Request) -> List[Any]:
        return [
            TotalAdmin(),
            TotalFile(),
            TotalImage(),
            TotalLog(),
            SystemInfo(),
            TeamInfo(),
        ]



from typing import Any, Optional
from .base import Base


class ImagePicker(Base):

    component: str = "imagePickerField"
    """
    组件名称
    """

    default_value: Optional[Any] = None
    """
    默认的选中项
    """

    disabled: bool = False
    """
    整组失效，默认 False
    """

    value: Optional[Any] = None
    """
    指定选中项
    """

    mode: str = "single"
    """
    上传模式，单图或多图，single|multiple
    """

    button: str = "上传图片"
    """
    上传按钮标识
    """

    limit_num: int = 3
    """
    上传文件数量限制
    """

    size: str = ""
    """
    按钮大小，default、small
    """

    def set_mode(self, v):
        """
        验证上传模式是否合法。

        Args:
            v (str): 上传模式。

        Returns:
            str: 合法的上传模式。

        Raises:
            ValueError: 当上传模式不合法时抛出异常。
        """
        if v == "s":
            v = "single"
        if v == "m":
            v = "multiple"
        limits = ["single", "multiple"]
        if v not in limits:
            raise ValueError("argument must be in 'single', 'multiple'!")
        self.mode = v
        return v

    def set_limit_num(self, limit_num: int):
        """
        设置上传文件数量限制。

        Args:
            limit_num (int): 上传文件数量限制值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.limit_num = limit_num
        return self

    def set_button(self, text: str):
        """
        设置上传按钮的标题。

        Args:
            text (str): 按钮标题。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.button = text
        return self

    def set_size(self, size: str):
        """
        设置按钮大小。

        Args:
            size (str): 按钮大小，如 'default', 'small'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.size = size
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self



from typing import Any, Dict, Optional

from fastapi import Request
from tortoise.models import Model


class Search:
    """基础搜索字段配置类"""

    # 字段名
    column: str = ""

    # 字段标题
    name: str = ""

    # 组件名称
    component: str = "textField"

    # 接口
    api: str = ""

    def __init__(self, column: str = "", name: str = ""):
        self.name = name
        self.column = column

    def get_column(self, search: Any) -> str:
        """
        获取数据库字段名。若未设置 column，则自动从类名推断。
        示例：ProductNameSearch -> product_name
        """
        if not self.column:
            class_name = type(search).__name__
            clean_name = class_name.replace("Search", "")
            return clean_name.lower()
        return self.column

    def get_name(self) -> str:
        """获取字段显示名称"""
        return self.name

    def get_component(self) -> str:
        """获取前端组件名称"""
        return self.component

    def get_api(self) -> str:
        """获取关联 API 接口地址"""
        return self.api

    def get_default(self) -> Any:
        """获取默认值"""
        return True

    def apply(self, request: Request, query: Model, value: Any) -> Model:
        """执行查询逻辑，子类可重写此方法"""
        return query

    def options(self, request: Request) -> Optional[Any]:
        """扩展属性选项"""
        return None

    def load(self, request: Request) -> Dict[str, str]:
        """
        单向联动配置，返回示例：
        {"field": "you_want_load_field", "api": "/api/admin/resource/action/select-options"}
        """
        return {}



from typing import Union, List, Dict, Optional
from ..component import Component

class Space(Component):
    # 组件属性定义
    component: str = "space"
    align: Optional[str] = None
    direction: Optional[str] = None
    size: Optional[str] = "small"  # 默认值为 small
    split: Optional[str] = None
    wrap: Optional[bool] = False  # 默认不自动换行
    body: Optional[Union[str, List]] = None  # body 可以是字符串或列表
    style: Optional[Dict[str, str]] = None  # 样式设置，字典形式

    def set_style(self, style: Dict[str, str]) -> "Component":
        """
        设置组件样式
        
        :param style: 样式字典
        :return: 返回当前组件实例
        """
        self.style = style
        return self

    def set_align(self, align: str) -> "Component":
        """
        设置对齐方式
        
        :param align: 对齐方式 (例如 'left', 'center', 'right')
        :return: 返回当前组件实例
        """
        self.align = align
        return self

    def set_direction(self, direction: str) -> "Component":
        """
        设置间距方向
        
        :param direction: 间距方向 (例如 'vertical', 'horizontal')
        :return: 返回当前组件实例
        """
        self.direction = direction
        return self

    def set_size(self, size: str) -> "Component":
        """
        设置间距大小
        
        :param size: 间距大小 (例如 'small', 'medium', 'large')
        :return: 返回当前组件实例
        """
        self.size = size
        return self

    def set_split(self, split: str) -> "Component":
        """
        设置拆分卡片的方向 (vertical | horizontal)
        
        :param split: 拆分方向
        :return: 返回当前组件实例
        """
        self.split = split
        return self

    def set_wrap(self, wrap: bool) -> "Component":
        """
        设置是否自动换行，仅在 horizontal 时有效
        
        :param wrap: 是否自动换行
        :return: 返回当前组件实例
        """
        self.wrap = wrap
        return self

    def set_body(self, body: Union[str, List]) -> "Component":
        """
        设置容器控件里面的内容
        
        :param body: 内容，可以是字符串或列表
        :return: 返回当前组件实例
        """
        self.body = body
        return self


import json
from typing import Any

from tortoise.models import Model
from tortoise.queryset import QuerySet

from quark import Message, Request

from ..performs_queries import PerformsQueries
from ..performs_validation import PerformsValidation


class UpdateRequest:

    # 请求对象
    request: Request

    # 资源对象
    resource: Any

    # 模型
    model: Model

    # 查询对象
    query: QuerySet

    # 字段列表
    fields: list

    def __init__(
        self,
        request: Request,
        resource: Any,
        model: Model,
        query: QuerySet,
        fields: list,
    ):
        self.request = request
        self.resource = resource
        self.model = model
        self.query = query
        self.fields = fields

    async def handle(self) -> Any:
        data = await self.request.json()

        # 验证参数合法性
        if not data.get("id"):
            return Message.error("参数错误")

        # 验证数据合法性
        try:
            await PerformsValidation(
                request=self.request, fields=self.fields
            ).validator_for_update(data)
        except Exception as e:
            return Message.error(str(e))

        # 保存前回调
        try:
            data = await self.resource.before_saving(self.request, data)
        except Exception as e:
            return Message.error(str(e))

        # 重组数据
        new_data = {}
        model_fields = set(self.model._meta.fields_map.keys())

        for k, v in data.items():
            # 只处理模型中存在的字段
            if k in model_fields:
                # 处理特殊数据类型
                if isinstance(v, (list, dict)):
                    new_data[k] = json.dumps(v, ensure_ascii=False)
                else:
                    new_data[k] = v

        query = await PerformsQueries(self.request).build_update_query(self.query)

        # 删除id，否则无法更新
        del new_data["id"]

        # 执行更新
        try:
            await query.filter(id=data["id"]).update(**new_data)
        except Exception as e:
            return Message.error(str(e))

        # 保存后回调
        try:
            await self.resource.after_saved(self.request, data["id"], data, query)
        except Exception as e:
            return Message.error(str(e))

        return await self.resource.after_saved_redirect_to(
            self.request, data["id"], data
        )



from typing import Any, List

from tortoise.queryset import QuerySet

from quark import Message, Request
from quark.template.action import Action


class BatchDeleteRole(Action):

    def __init__(self):
        self.name = "批量删除"
        self.type = "primary"
        self.size = "small"
        self.reload = "table"
        self.batch = True
        self.danger = True
        self.with_confirm(
            "确定要删除吗？", "删除后数据将无法恢复，请谨慎操作！", "modal"
        )
        self.set_only_on_index(True)

    def get_api_params(self) -> List[str]:
        return ["id"]

    async def handle(self, request: Request, query: QuerySet) -> Any:
        id_param = request.query_params.get("id")

        if not id_param:
            return Message.error("参数错误")

        try:
            ids = id_param.split(",")
            id_list = [int(i) for i in ids]

            # 删除角色
            await query.filter(id__in=id_list).delete()

            # 清理 casbin 权限
            # casbin_service = CasbinService()
            # for id_int in id_list:
            #     await casbin_service.remove_role_menu_and_permissions(id_int)

            return Message.success("操作成功")
        except Exception as e:
            return Message.error(str(e))



from datetime import datetime

from tortoise import fields
from tortoise.models import Model

from ..utils import hash_password


class User(Model):
    id = fields.IntField(pk=True)
    username = fields.CharField(max_length=20)
    nickname = fields.CharField(max_length=200)
    sex = fields.IntField(default=1)
    email = fields.CharField(max_length=50)
    phone = fields.CharField(max_length=11)
    password = fields.CharField(max_length=255)
    avatar = fields.CharField(max_length=1000, null=True)
    department_id = fields.IntField(null=True)
    position_ids = fields.CharField(max_length=1000, null=True)
    last_login_ip = fields.CharField(max_length=255, null=True)
    last_login_time = fields.DatetimeField(null=True, default=datetime.now())
    wx_openid = fields.CharField(max_length=255, null=True)
    wx_unionid = fields.CharField(max_length=255, null=True)
    status = fields.IntField(default=1)
    created_at = fields.DatetimeField(auto_now_add=True)
    updated_at = fields.DatetimeField(auto_now=True)
    deleted_at = fields.DatetimeField(null=True)

    class Meta(Model.Meta):
        table = "users"
        unique_together = (
            ("username",),
            ("email",),
            ("phone",),
        )

    @classmethod
    async def seeder(cls):
        # 检查是否已存在超级管理员
        existing = await cls.get_or_none(username="administrator")
        if existing:
            return

        await cls.create(
            username="administrator",
            nickname="超级管理员",
            email="admin@yourweb.com",
            phone="10086",
            password=hash_password("123456"),
            sex=1,
            department_id=1,
            status=1,
            last_login_time=datetime.utcnow(),
        )



from typing import Any, Dict, Optional

from pydantic import Field, model_validator

from ..component import Component


class PageHeader(Component):
    component: str = Field(default="pageHeader")
    avatar: Optional[Any] = Field(None, description="标题栏旁的头像")
    back_icon: Optional[Any] = Field(
        None, description="自定义 back icon ，如果为 false 不渲染 back icon"
    )
    breadcrumb: Optional[Any] = Field(None, description="面包屑的配置")
    breadcrumb_render: Optional[Any] = Field(None, description="自定义面包屑区域的内容")
    extra: Optional[Any] = Field(None, description="操作区，位于 title 行的行尾")
    footer: Optional[Any] = Field(
        None, description="PageHeader 的页脚，一般用于渲染 TabBar"
    )
    ghost: bool = Field(True, description="pageHeader 的类型，将会改变背景颜色")
    sub_title: Optional[str] = Field(None, description="自定义的二级标题文字")
    tags: Optional[Any] = Field(None, description="title 旁的 tag 列表")
    title: Optional[str] = Field(None, description="自定义标题文字")

    @model_validator(mode="after")
    def init(self):
        self.set_key("pageHeader")
        return self

    def set_style(self, style: Dict[str, Any]):
        # 设置样式
        self.style = style
        return self

    def set_avatar(self, avatar: Any):
        # 设置头像
        self.avatar = avatar
        return self

    def set_back_icon(self, back_icon: Any):
        # 设置 back icon
        self.back_icon = back_icon
        return self

    def set_breadcrumb(self, breadcrumb: Any):
        # 设置面包屑配置
        self.breadcrumb = breadcrumb
        return self

    def set_breadcrumb_render(self, breadcrumb_render: Any):
        # 设置自定义面包屑区域的内容
        self.breadcrumb_render = breadcrumb_render
        return self

    def set_extra(self, extra: Any):
        # 设置操作区
        self.extra = extra
        return self

    def set_footer(self, footer: Any):
        # 设置页脚
        self.footer = footer
        return self

    def set_ghost(self, ghost: bool):
        # 设置 pageHeader 类型
        self.ghost = ghost
        return self

    def set_sub_title(self, sub_title: str):
        # 设置二级标题
        self.sub_title = sub_title
        return self

    def set_tags(self, tags: Any):
        # 设置 tag 列表
        self.tags = tags
        return self

    def set_title(self, title: str):
        # 设置标题
        self.title = title
        return self



from fastapi import Request
from fastapi.encoders import jsonable_encoder
from fastapi.responses import JSONResponse, Response
from starlette.middleware.base import BaseHTTPMiddleware

from quark import Message
from quark.models import ActionLog
from quark.services import ActionLogService, AuthService


# 自定义中间件
class Middleware(BaseHTTPMiddleware):

    async def dispatch(self, request: Request, call_next) -> Response:
        response = await call_next(request)
        url_path = request.url.path
        route = request.scope.get("route")
        route_path = route.path if route else url_path

        # 登录路由
        login_routes = [
            {
                "path": "/api/admin/auth/{resource}/index",
                "method": "GET",
            },
            {
                "path": "/api/admin/auth/{resource}/captcha",
                "method": "GET",
            },
            {
                "path": "/api/admin/auth/{resource}/login",
                "method": "POST",
            },
            {
                "path": "/api/admin/auth/{resource}/handle",
                "method": "GET",
            },
        ]

        # 判断是否在登录路由中
        in_login_route = any(v["path"] == route_path for v in login_routes)
        if in_login_route:
            return response

        # 排除非后台路由
        if "api/admin" not in url_path:
            return response

        # 获取管理员信息
        auth_service = AuthService(request)
        try:
            admin_info = await auth_service.get_current_admin()
        except Exception as e:
            return JSONResponse(
                status_code=401,
                content=jsonable_encoder(Message.error(str(e)), exclude_none=True),
            )

        # 权限验证
        if admin_info.id != 1:
            results = []
            for path in [route_path, url_path]:
                try:
                    result = await auth_service.check_permission(path, request.method)
                    results.append(result)
                except Exception as e:
                    return JSONResponse(
                        status_code=500,
                        content=jsonable_encoder(
                            Message.error(str(e)), exclude_none=True
                        ),
                    )

            if not any(results):
                return JSONResponse(
                    status_code=403,
                    content=jsonable_encoder(
                        Message.error("403 Forbidden"), exclude_none=True
                    ),
                )

        # 记录操作日志
        await ActionLogService().insert_get_id(
            ActionLog(
                uid=admin_info.id,
                username=admin_info.username,
                url=url_path,
                ip=request.client.host if request.client else "",
                type="ADMIN",
                remark="",
            )
        )

        return response
